"""
Snowflake Notification Integration API.

The Snowflake Notification Integration API is a REST API that you can use to access, update, and perform certain actions on Notification Integration resource in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any, Dict, Optional

from pydantic import ConfigDict, StrictStr

from snowflake.core.notification_integration._generated.models.notification_hook import (
    NotificationHook,
)
from snowflake.core.notification_integration._generated.models.webhook_secret import WebhookSecret, WebhookSecretModel


class NotificationWebhook(NotificationHook):
    """A model object representing the NotificationWebhook resource.

    Constructs an object of type NotificationWebhook with the provided properties.

    Parameters
    __________
    webhook_url : str
        The URL for the webhook. The URL must use the https:// protocol.
    webhook_secret : WebhookSecret, optional

    webhook_body_template : str, optional
        A template for the body of the HTTP request to send for the notification.
    webhook_headers : str, optional
        A list of HTTP headers and values to include in the HTTP request for the webhook.
    """

    webhook_url: StrictStr

    webhook_secret: Optional[WebhookSecret] = None

    webhook_body_template: Optional[StrictStr] = None

    webhook_headers: Optional[Dict[str, StrictStr]] = None

    __properties = ["type"]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> NotificationWebhook:
        """Create an instance of NotificationWebhook from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        # override the default output from pydantic by calling `to_dict()` of webhook_secret
        if self.webhook_secret:
            _dict["webhook_secret"] = self.webhook_secret.to_dict()

        _dict["type"] = NotificationHook.get_child_model_discriminator_value("NotificationWebhook")

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> NotificationWebhook:
        """Create an instance of NotificationWebhook from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return NotificationWebhook.model_validate(obj)

        _obj = NotificationWebhook.model_validate(
            {
                "webhook_url": obj.get("webhook_url"),
                "webhook_secret": WebhookSecret.from_dict(obj.get("webhook_secret"))
                if obj.get("webhook_secret") is not None
                else None,
                "webhook_body_template": obj.get("webhook_body_template"),
                "webhook_headers": obj.get("webhook_headers"),
            }
        )

        return _obj


class NotificationWebhookModel(NotificationHook):
    def __init__(
        self,
        webhook_url: str,
        # optional properties
        webhook_secret: Optional[WebhookSecret] = None,
        webhook_body_template: Optional[str] = None,
        webhook_headers: Optional[str] = None,
    ):
        """A model object representing the NotificationWebhook resource.

        Constructs an object of type NotificationWebhook with the provided properties.

        Parameters
        __________
        webhook_url : str
            The URL for the webhook. The URL must use the https:// protocol.
        webhook_secret : WebhookSecret, optional

        webhook_body_template : str, optional
            A template for the body of the HTTP request to send for the notification.
        webhook_headers : str, optional
            A list of HTTP headers and values to include in the HTTP request for the webhook.
        """
        super().__init__()
        self.webhook_url = webhook_url
        self.webhook_secret = webhook_secret
        self.webhook_body_template = webhook_body_template
        self.webhook_headers = webhook_headers

    __properties = ["type"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return NotificationWebhook(
            webhook_url=self.webhook_url,
            webhook_secret=self.webhook_secret._to_model() if self.webhook_secret is not None else None,
            webhook_body_template=self.webhook_body_template,
            webhook_headers=self.webhook_headers,
        )

    @classmethod
    def _from_model(cls, model) -> NotificationWebhookModel:
        return NotificationWebhookModel(
            webhook_url=model.webhook_url,
            webhook_secret=WebhookSecretModel._from_model(model.webhook_secret) if model.webhook_secret else None,
            webhook_body_template=model.webhook_body_template,
            webhook_headers=model.webhook_headers,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a NotificationWebhook.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the NotificationWebhook object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> NotificationWebhookModel:
        """Create an instance of NotificationWebhook from a dict.

        This method constructs a NotificationWebhook object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        NotificationWebhook
            A NotificationWebhook object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(NotificationWebhook.from_dict(obj))


NotificationWebhook._model_class = NotificationWebhookModel



import re

from typing import Any, Optional

from snowflake.core._utils import check_version_gte, check_version_lte
from snowflake.core.version import __version__


MINIMUM_SUPPORT_VERSION = "minimumSupportedVersion"
MINIMUM_NEARING_END_OF_SUPPORT_VERSION = "minimumNearingEndOfSupportVersion"
RECOMMENDED_VERSION = "recommendedVersion"


class ClientInfo:
    def __init__(self, client_support_info: Optional[dict[str, str]]) -> None:
        """Store the metadata about the client, and server configuration for the client.

        Parameters
        __________
            client_support_info: dict[str, str], optional
                A parsed (from JSON) dictionary of client support information.
        """
        self._client_support_info = None
        if self._validate_client_info(client_support_info):
            self._client_support_info = client_support_info

    @property
    def client_version(self) -> str:
        return __version__

    @property
    def minimum_supported_version(self) -> Optional[str]:
        if self._client_support_info is not None and self._client_support_info[MINIMUM_SUPPORT_VERSION] != "":
            return self._client_support_info[MINIMUM_SUPPORT_VERSION]

        return None

    @property
    def end_of_support_version(self) -> Optional[str]:
        if (
            self._client_support_info is not None
            and self._client_support_info[MINIMUM_NEARING_END_OF_SUPPORT_VERSION] != ""
        ):
            return self._client_support_info[MINIMUM_NEARING_END_OF_SUPPORT_VERSION]

        return None

    @property
    def recommended_version(self) -> Optional[str]:
        if self._client_support_info is not None and self._client_support_info[RECOMMENDED_VERSION] != "":
            return self._client_support_info[RECOMMENDED_VERSION]

        return None

    def version_is_supported(self) -> bool:
        # If the minimum supported version is empty, then we assume that the client is supported
        if self._client_support_info is None or self.minimum_supported_version is None:
            return True

        return check_version_gte(self.client_version, self.minimum_supported_version)

    def version_is_nearing_end_of_support(self) -> bool:
        if self._client_support_info is None or self.end_of_support_version is None:
            return False

        return check_version_lte(self.client_version, self.end_of_support_version)

    @staticmethod
    def _validate_client_info(client_support_info: Optional[dict[str, Any]]) -> bool:
        # If the client support information is None, then we assume that the client is supported
        if client_support_info is not None:
            # Check for the clientId, and that it is "PyCore"
            if "clientId" not in client_support_info or client_support_info["clientId"] != "PyCore":
                return False

            # Regex pattern for valid version strings, which can be empty
            pattern = re.compile(r"^(\d+\.\d+\.\d+)?$")

            try:
                if (
                    MINIMUM_SUPPORT_VERSION not in client_support_info
                    or pattern.match(client_support_info[MINIMUM_SUPPORT_VERSION]) is None
                ):
                    return False

                if (
                    MINIMUM_NEARING_END_OF_SUPPORT_VERSION not in client_support_info
                    or pattern.match(client_support_info[MINIMUM_NEARING_END_OF_SUPPORT_VERSION]) is None
                ):
                    return False

                if (
                    RECOMMENDED_VERSION not in client_support_info
                    or pattern.match(client_support_info[RECOMMENDED_VERSION]) is None
                ):
                    return False

            except TypeError:
                # This can happen if the Dict does not have string values when trying to match the regex
                return False

        return True



"""Manages Snowflake ManagedAccounts.

Example:
    >>> managed_account_collection = root.managed_accounts
    >>> managed_account = ManagedAccount(
    ...     name="managed_account_name",
    ...     admin_name = "admin"
    ...     admin_password = 'TestPassword1'
    ...     account_type = "READER"
    ...  )
    >>> managed_account_collection.create(managed_account)

Refer to :class:`snowflake.core.Root` to create the ``root``.
"""

from ._managed_account import ManagedAccount, ManagedAccountCollection, ManagedAccountResource


__all__ = ["ManagedAccount", "ManagedAccountCollection", "ManagedAccountResource"]



"""
Cortex Inference API.

OpenAPI 3.0 specification for the Cortex REST API  # noqa: E501

The version of the OpenAPI document: 0.1.0
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any, Optional

from pydantic import BaseModel, ConfigDict, Field, StrictInt
from typing_extensions import Annotated


class Usage(BaseModel):
    """A model object representing the Usage resource.

    Constructs an object of type Usage with the provided properties.

    Parameters
    __________
    prompt_tokens : int, optional
        Input token count.
    completion_tokens : int, optional
        Output token count.
    guard_tokens : int, optional
        Tokens used by cortex guard.
    total_tokens : int, optional
        Sum of all tokens.
    cache_read_input_tokens : int, optional
        reserved
    cache_write_input_tokens : int, optional
        reserved
    """

    prompt_tokens: Optional[StrictInt] = None

    completion_tokens: Optional[StrictInt] = None

    guard_tokens: Optional[StrictInt] = None

    total_tokens: Optional[StrictInt] = None

    cache_read_input_tokens: Optional[Annotated[int, Field(strict=True, ge=0)]] = None

    cache_write_input_tokens: Optional[Annotated[int, Field(strict=True, ge=0)]] = None

    __properties = [
        "prompt_tokens",
        "completion_tokens",
        "guard_tokens",
        "total_tokens",
        "cache_read_input_tokens",
        "cache_write_input_tokens",
    ]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Usage:
        """Create an instance of Usage from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        # set to None if cache_read_input_tokens (nullable) is None
        if self.cache_read_input_tokens is None:
            _dict["cache_read_input_tokens"] = None

        # set to None if cache_write_input_tokens (nullable) is None
        if self.cache_write_input_tokens is None:
            _dict["cache_write_input_tokens"] = None

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> Usage:
        """Create an instance of Usage from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return Usage.model_validate(obj)

        _obj = Usage.model_validate(
            {
                "prompt_tokens": obj.get("prompt_tokens"),
                "completion_tokens": obj.get("completion_tokens"),
                "guard_tokens": obj.get("guard_tokens"),
                "total_tokens": obj.get("total_tokens"),
                "cache_read_input_tokens": obj.get("cache_read_input_tokens"),
                "cache_write_input_tokens": obj.get("cache_write_input_tokens"),
            }
        )

        return _obj


class UsageModel:
    def __init__(
        self,  # optional properties
        prompt_tokens: Optional[int] = None,
        completion_tokens: Optional[int] = None,
        guard_tokens: Optional[int] = None,
        total_tokens: Optional[int] = None,
        cache_read_input_tokens: Optional[int] = None,
        cache_write_input_tokens: Optional[int] = None,
    ):
        """A model object representing the Usage resource.

        Constructs an object of type Usage with the provided properties.

        Parameters
        __________
        prompt_tokens : int, optional
            Input token count.
        completion_tokens : int, optional
            Output token count.
        guard_tokens : int, optional
            Tokens used by cortex guard.
        total_tokens : int, optional
            Sum of all tokens.
        cache_read_input_tokens : int, optional
            reserved
        cache_write_input_tokens : int, optional
            reserved
        """
        self.prompt_tokens = prompt_tokens
        self.completion_tokens = completion_tokens
        self.guard_tokens = guard_tokens
        self.total_tokens = total_tokens
        self.cache_read_input_tokens = cache_read_input_tokens
        self.cache_write_input_tokens = cache_write_input_tokens

    __properties = [
        "prompt_tokens",
        "completion_tokens",
        "guard_tokens",
        "total_tokens",
        "cache_read_input_tokens",
        "cache_write_input_tokens",
    ]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return Usage(
            prompt_tokens=self.prompt_tokens,
            completion_tokens=self.completion_tokens,
            guard_tokens=self.guard_tokens,
            total_tokens=self.total_tokens,
            cache_read_input_tokens=self.cache_read_input_tokens,
            cache_write_input_tokens=self.cache_write_input_tokens,
        )

    @classmethod
    def _from_model(cls, model) -> UsageModel:
        return UsageModel(
            prompt_tokens=model.prompt_tokens,
            completion_tokens=model.completion_tokens,
            guard_tokens=model.guard_tokens,
            total_tokens=model.total_tokens,
            cache_read_input_tokens=model.cache_read_input_tokens,
            cache_write_input_tokens=model.cache_write_input_tokens,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a Usage.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the Usage object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> UsageModel:
        """Create an instance of Usage from a dict.

        This method constructs a Usage object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        Usage
            A Usage object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(Usage.from_dict(obj))


Usage._model_class = UsageModel



import copy

from contextlib import suppress

import pytest as pytest

from snowflake.core.exceptions import NotFoundError
from tests.integ.utils import random_string

from ...utils import ensure_snowflake_version
from .conftest import test_view_template


@pytest.fixture(scope="module")
def views_extended(views, temp_table, snowflake_version):
    ensure_snowflake_version(snowflake_version, "8.33.0")

    names_list = []
    for _ in range(5):
        names_list.append(random_string(10, "test_view_iter_a_"))
    for _ in range(7):
        names_list.append(random_string(10, "test_view_iter_b_"))
    for _ in range(3):
        names_list.append(random_string(10, "test_view_iter_c_"))

    try:
        for view_name in names_list:
            view_temp = copy.deepcopy(test_view_template)
            view_temp.name = view_name
            view_temp.query = f"SELECT * FROM {temp_table.name}"
            views.create(view_temp)

        yield views
    finally:
        for view_name in names_list:
            with suppress(NotFoundError):
                views[view_name].drop()


def test_iter_raw(views_extended):
    assert len(list(views_extended.iter())) >= 15


def test_iter_like(views_extended):
    assert len(list(views_extended.iter(like="test_view_iter_"))) == 0
    assert len(list(views_extended.iter(like="test_view_iter_a_%%"))) == 5
    assert len(list(views_extended.iter(like="test_view_iter_b_%%"))) == 7
    assert len(list(views_extended.iter(like="test_view_iter_c_%%"))) == 3


def test_iter_show_limit(views_extended):
    assert len(list(views_extended.iter(like="test_view_iter_a_%%"))) == 5
    assert len(list(views_extended.iter(like="test_view_iter_a_%%", show_limit=2))) == 2
    assert len(list(views_extended.iter(show_limit=2))) == 2


def test_iter_starts_with(views_extended):
    assert len(list(views_extended.iter(starts_with="test_view_iter_a_".upper()))) == 5


def test_iter_from_name(views_extended):
    assert len(list(views_extended.iter(from_name="test_view_iter_b_"))) >= 10


def test_iter_deep(views_extended):
    for v in views_extended.iter(show_limit=10):
        assert v.columns == []

    for v in views_extended.iter(show_limit=10, deep=True):
        assert len(v.columns) > 0



from snowflake.core.tag import Tag


test_tag_template = Tag(
    name="to_be_set",
    allowed_values=["value1", "value2", "value3"],
    comment="Test tag",
    # propagate and on_conflict are settable only on ENTERPRISE level accounts
)

test_tag_minimal_template = Tag(
    name="to_be_set",
)



"""
Snowflake Role API.

The Snowflake Role API is a REST API that you can use to access, update, and perform certain actions on Role resource in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re

from datetime import datetime
from typing import Any, Optional

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr, field_validator
from typing_extensions import Annotated


class Role(BaseModel):
    """A model object representing the Role resource.

    Constructs an object of type Role with the provided properties.

    Parameters
    __________
    name : str
        Name of the role.
    comment : str, optional
        Comment of the role.
    created_on : datetime, optional
        Date and time when the role was created — **Read-only:** *any user-provided value will be ignored.*
    owner : str, optional
        Specifies the role that owns this role — **Read-only:** *any user-provided value will be ignored.*
    is_default : bool, optional
        Specifies whether the role being fetched is the user's default role — **Read-only:** *any user-provided value will be ignored.*
    is_current : bool, optional
        Specifies whether the role being fetched is the user's current role — **Read-only:** *any user-provided value will be ignored.*
    is_inherited : bool, optional
        Specifies whether the role used to run the command inherits the specified role — **Read-only:** *any user-provided value will be ignored.*
    assigned_to_users : int, optional
        The number of users to whom this role has been assigned — **Read-only:** *any user-provided value will be ignored.*
    granted_to_roles : int, optional
        The number of roles to which this role has been granted — **Read-only:** *any user-provided value will be ignored.*
    granted_roles : int, optional
        The number of roles that have been granted to this role — **Read-only:** *any user-provided value will be ignored.*
    """

    name: Annotated[str, Field(strict=True)]

    comment: Optional[StrictStr] = None

    created_on: Optional[datetime] = None

    owner: Optional[Annotated[str, Field(strict=True)]] = None

    is_default: Optional[StrictBool] = None

    is_current: Optional[StrictBool] = None

    is_inherited: Optional[StrictBool] = None

    assigned_to_users: Optional[StrictInt] = None

    granted_to_roles: Optional[StrictInt] = None

    granted_roles: Optional[StrictInt] = None

    __properties = [
        "name",
        "comment",
        "created_on",
        "owner",
        "is_default",
        "is_current",
        "is_inherited",
        "assigned_to_users",
        "granted_to_roles",
        "granted_roles",
    ]

    @field_validator("name")
    def name_validate_regular_expression(cls, v):
        if not re.match(r"""^\"([^\"]|\"\")+\"|[a-zA-Z_][a-zA-Z0-9_$]*$""", v):
            raise ValueError(r"""must validate the regular expression /^"([^"]|"")+"|[a-zA-Z_][a-zA-Z0-9_$]*$/""")
        return v

    @field_validator("owner")
    def owner_validate_regular_expression(cls, v):
        if v is None:
            return v
        if not re.match(r"""^\"([^\"]|\"\")+\"|[a-zA-Z_][a-zA-Z0-9_$]*$""", v):
            raise ValueError(r"""must validate the regular expression /^"([^"]|"")+"|[a-zA-Z_][a-zA-Z0-9_$]*$/""")
        return v

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Role:
        """Create an instance of Role from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update(
                {
                    "created_on",
                    "owner",
                    "is_default",
                    "is_current",
                    "is_inherited",
                    "assigned_to_users",
                    "granted_to_roles",
                    "granted_roles",
                }
            )

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> Role:
        """Create an instance of Role from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return Role.model_validate(obj)

        _obj = Role.model_validate(
            {
                "name": obj.get("name"),
                "comment": obj.get("comment"),
                "created_on": obj.get("created_on"),
                "owner": obj.get("owner"),
                "is_default": obj.get("is_default"),
                "is_current": obj.get("is_current"),
                "is_inherited": obj.get("is_inherited"),
                "assigned_to_users": obj.get("assigned_to_users"),
                "granted_to_roles": obj.get("granted_to_roles"),
                "granted_roles": obj.get("granted_roles"),
            }
        )

        return _obj


class RoleModel:
    def __init__(
        self,
        name: str,
        # optional properties
        comment: Optional[str] = None,
        created_on: Optional[datetime] = None,
        owner: Optional[str] = None,
        is_default: Optional[bool] = None,
        is_current: Optional[bool] = None,
        is_inherited: Optional[bool] = None,
        assigned_to_users: Optional[int] = None,
        granted_to_roles: Optional[int] = None,
        granted_roles: Optional[int] = None,
    ):
        """A model object representing the Role resource.

        Constructs an object of type Role with the provided properties.

        Parameters
        __________
        name : str
            Name of the role.
        comment : str, optional
            Comment of the role.
        created_on : datetime, optional
            Date and time when the role was created.
        owner : str, optional
            Specifies the role that owns this role.
        is_default : bool, optional
            Specifies whether the role being fetched is the user's default role.
        is_current : bool, optional
            Specifies whether the role being fetched is the user's current role.
        is_inherited : bool, optional
            Specifies whether the role used to run the command inherits the specified role.
        assigned_to_users : int, optional
            The number of users to whom this role has been assigned.
        granted_to_roles : int, optional
            The number of roles to which this role has been granted.
        granted_roles : int, optional
            The number of roles that have been granted to this role.
        """
        self.name = name
        self.comment = comment
        self.created_on = created_on
        self.owner = owner
        self.is_default = is_default
        self.is_current = is_current
        self.is_inherited = is_inherited
        self.assigned_to_users = assigned_to_users
        self.granted_to_roles = granted_to_roles
        self.granted_roles = granted_roles

    __properties = [
        "name",
        "comment",
        "created_on",
        "owner",
        "is_default",
        "is_current",
        "is_inherited",
        "assigned_to_users",
        "granted_to_roles",
        "granted_roles",
    ]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return Role(
            name=self.name,
            comment=self.comment,
            created_on=self.created_on,
            owner=self.owner,
            is_default=self.is_default,
            is_current=self.is_current,
            is_inherited=self.is_inherited,
            assigned_to_users=self.assigned_to_users,
            granted_to_roles=self.granted_to_roles,
            granted_roles=self.granted_roles,
        )

    @classmethod
    def _from_model(cls, model) -> RoleModel:
        return RoleModel(
            name=model.name,
            comment=model.comment,
            created_on=model.created_on,
            owner=model.owner,
            is_default=model.is_default,
            is_current=model.is_current,
            is_inherited=model.is_inherited,
            assigned_to_users=model.assigned_to_users,
            granted_to_roles=model.granted_to_roles,
            granted_roles=model.granted_roles,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a Role.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the Role object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> RoleModel:
        """Create an instance of Role from a dict.

        This method constructs a Role object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        Role
            A Role object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(Role.from_dict(obj))


Role._model_class = RoleModel



from collections.abc import Iterator
from concurrent.futures import Future
from typing import TYPE_CHECKING, Annotated, Literal, Optional, Union, overload

from pydantic import Field, StrictStr

from snowflake.core import FQN, PollingOperation
from snowflake.core._common import (
    Clone,
    CreateMode,
    PointOfTime,
    SchemaObjectCollectionParent,
    SchemaObjectReferenceMixin,
)
from snowflake.core._generated.api_client import StoredProcApiClient
from snowflake.core._operation import PollingOperations
from snowflake.core._utils import (
    tag_assignment_to_tag_tuple,
    tag_resource_to_tag_reference,
    tag_tuple_to_tag_assignment,
)
from snowflake.core.tag import TagResource, TagValue

from .._internal.telemetry import api_telemetry
from ._generated import ConvertToManagedIcebergTableRequest, RefreshIcebergTableRequest, SuccessResponse
from ._generated.api import IcebergTableApi
from ._generated.models.iceberg_table import IcebergTable
from ._generated.models.iceberg_table_as_select import IcebergTableAsSelect
from ._generated.models.iceberg_table_clone import IcebergTableClone
from ._generated.models.iceberg_table_from_aws_glue_catalog import IcebergTableFromAWSGlueCatalog
from ._generated.models.iceberg_table_from_delta import IcebergTableFromDelta
from ._generated.models.iceberg_table_from_iceberg_files import IcebergTableFromIcebergFiles
from ._generated.models.iceberg_table_from_iceberg_rest import IcebergTableFromIcebergRest
from ._generated.models.iceberg_table_like import IcebergTableLike
from ._generated.models.point_of_time import PointOfTime as IcebergTablePointOfTime
from ._generated.models.tag_assignment import TagAssignment
from ._generated.models.tag_reference import TagReference


if TYPE_CHECKING:
    from snowflake.core.schema import SchemaResource


class IcebergTableCollection(SchemaObjectCollectionParent["IcebergTableResource"]):
    """Represents the collection operations on the Snowflake Iceberg table resource.

    With this collection, you can create, iterate through, and search for Iceberg tables that you
    have access to in the current context.

    Examples
    ________
    Creating an IcebergTable instance:

    >>> iceberg_tables = root.databases["my_db"].schemas["my_schema"].iceberg_tables
    >>> new_iceberg_table = IcebergTable(
    ...     name="name", columns=[IcebergTableColumn(name="col1", datatype="string")]
    ... )
    >>> iceberg_tables.create(new_iceberg_table)
    """

    def __init__(self, schema: "SchemaResource"):
        super().__init__(schema, IcebergTableResource)
        self._api = IcebergTableApi(
            root=self.root, resource_class=self._ref_class, sproc_client=StoredProcApiClient(root=self.root)
        )

    @overload
    @api_telemetry
    def create(
        self,
        iceberg_table: IcebergTable,
        *,
        copy_grants: Optional[bool] = False,
        mode: CreateMode = CreateMode.error_if_exists,
    ) -> "IcebergTableResource": ...

    @overload
    @api_telemetry
    def create(
        self,
        iceberg_table: IcebergTable,
        *,
        as_select: Optional[str] = None,
        copy_grants: Optional[bool] = False,
        mode: CreateMode = CreateMode.error_if_exists,
    ) -> "IcebergTableResource": ...

    @overload
    @api_telemetry
    def create(
        self,
        iceberg_table: IcebergTable,
        *,
        like: str,
        copy_grants: Optional[bool] = False,
        mode: CreateMode = CreateMode.error_if_exists,
    ) -> "IcebergTableResource": ...

    @overload
    @api_telemetry
    def create(
        self,
        iceberg_table: IcebergTable,
        *,
        from_aws_glue_catalog: Literal[True] = True,
        mode: CreateMode = CreateMode.error_if_exists,
    ) -> "IcebergTableResource": ...

    @overload
    @api_telemetry
    def create(
        self,
        iceberg_table: IcebergTable,
        *,
        from_delta: Literal[True] = True,
        mode: CreateMode = CreateMode.error_if_exists,
    ) -> "IcebergTableResource": ...

    @overload
    @api_telemetry
    def create(
        self,
        iceberg_table: IcebergTable,
        *,
        from_iceberg_files: Literal[True] = True,
        mode: CreateMode = CreateMode.error_if_exists,
    ) -> "IcebergTableResource": ...

    @overload
    @api_telemetry
    def create(
        self,
        iceberg_table: IcebergTable,
        *,
        from_iceberg_rest: Literal[True] = True,
        mode: CreateMode = CreateMode.error_if_exists,
    ) -> "IcebergTableResource": ...

    @overload
    @api_telemetry
    def create(
        self,
        iceberg_table: IcebergTable,
        *,
        clone_iceberg_table: Union[str, Clone],
        copy_grants: Optional[bool] = False,
        mode: CreateMode = CreateMode.error_if_exists,
    ) -> "IcebergTableResource": ...

    @api_telemetry
    def create(
        self,
        iceberg_table: IcebergTable,
        *,
        as_select: Optional[str] = None,
        like: Optional[str] = None,
        from_aws_glue_catalog: Optional[bool] = False,
        from_delta: Optional[bool] = False,
        from_iceberg_files: Optional[bool] = False,
        from_iceberg_rest: Optional[bool] = False,
        clone_iceberg_table: Optional[Union[str, Clone]] = None,
        copy_grants: Optional[bool] = False,
        mode: CreateMode = CreateMode.error_if_exists,
    ) -> "IcebergTableResource":
        """Create an Iceberg table in Snowflake.

        There are multiple ways to create an Iceberg table:
            * by building from scratch
            * by creating from select query
            * by creating it from AWS Glue Catalog or Delta catalog
            * by creating it from files
            * by creating it from REST API
            * by cloning existing table

        Parameters
        __________
        iceberg_table: Union[IcebergTable, str]
            The new Iceberg table's name.
        as_select: str
            Creates an Iceberg table using a select query.
        like: str
            Create a new Iceberg table like the specified one, but empty.
        from_aws_glue_catalog: IcebergTableFromAwsGlueCatalog
            Creates an Iceberg table from AWS Glue Catalog.
        from_delta: IcebergTableFromDelta
            Creates an Iceberg table from Delta.
        from_iceberg_files: IcebergTableFromIcebergFiles
            Creates an Iceberg table using Iceberg files in object storage (external cloud storage).
        from_iceberg_rest: IcebergTableFromIcebergRest
            Creates an Iceberg in Iceberg REST catalog.
        clone_iceberg_table: str or Clone object
            The name of Iceberg table to be cloned, or a ``Clone`` object which would contain the
            name of the Iceberg table with support to clone at a specific time.
        copy_grants: bool, optional
            Copy grants when resource is created.
        mode: CreateMode, optional
            One of the following enum values:

            ``CreateMode.error_if_exists``: Throw an :class:`snowflake.core.exceptions.ConflictError`
            if the Iceberg table already exists in Snowflake.  Equivalent to SQL ``create Iceberg table <name> ...``.

            ``CreateMode.or_replace``: Replace if the Iceberg table already exists in Snowflake. Equivalent to SQL
            ``create or replace Iceberg table <name> ...``.

            ``CreateMode.if_not_exists``: Do nothing if the Iceberg table already exists in Snowflake.
            Equivalent to SQL ``create Iceberg table <name> if not exists...``

            Default is ``CreateMode.error_if_exists``.

        Examples
        ________
        Creating an Iceberg table, replacing any existing Iceberg table with the same name:

        >>> iceberg_tables = root.databases["my_db"].schemas["my_schema"].iceberg_tables
        >>> new_iceberg_table = IcebergTable(
        ...     name="name", columns=[IcebergTableColumn(name="col1", datatype="string")]
        ... )
        >>> iceberg_tables.create(new_iceberg_table, mode=CreateMode.or_replace)

        Cloning an Iceberg table instance:

        >>> iceberg_tables = root.databases["my_db"].schemas["my_schema"].iceberg_tables
        >>> iceberg_tables.create(
        ...     IcebergTable(name="new_table"),
        ...     clone_iceberg_table="iceberg_table_name_to_be_cloned",
        ...     mode=CreateMode.if_not_exists,
        ... )
        """
        self._create(
            iceberg_table=iceberg_table,
            as_select=as_select,
            like=like,
            from_aws_glue_catalog=from_aws_glue_catalog,
            from_delta=from_delta,
            from_iceberg_files=from_iceberg_files,
            from_iceberg_rest=from_iceberg_rest,
            clone_iceberg_table=clone_iceberg_table,
            copy_grants=copy_grants,
            mode=mode,
            async_req=False,
        )
        return IcebergTableResource(iceberg_table.name, self)

    @api_telemetry
    def create_async(
        self,
        iceberg_table: IcebergTable,
        *,
        as_select: Optional[str] = None,
        like: Optional[str] = None,
        from_aws_glue_catalog: Optional[bool] = False,
        from_delta: Optional[bool] = False,
        from_iceberg_files: Optional[bool] = False,
        from_iceberg_rest: Optional[bool] = False,
        clone_iceberg_table: Optional[Union[str, Clone]] = None,
        copy_grants: Optional[bool] = False,
        mode: CreateMode = CreateMode.error_if_exists,
    ) -> PollingOperation["IcebergTableResource"]:
        """An asynchronous version of :func:`create`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self._create(
            iceberg_table=iceberg_table,
            as_select=as_select,
            like=like,
            from_aws_glue_catalog=from_aws_glue_catalog,
            from_delta=from_delta,
            from_iceberg_files=from_iceberg_files,
            from_iceberg_rest=from_iceberg_rest,
            clone_iceberg_table=clone_iceberg_table,
            copy_grants=copy_grants,
            mode=mode,
            async_req=True,
        )
        return PollingOperation(future, lambda _: IcebergTableResource(iceberg_table.name, self))

    @api_telemetry
    def iter(
        self,
        *,
        like: Optional[StrictStr] = None,
        starts_with: Optional[StrictStr] = None,
        show_limit: Optional[Annotated[int, Field(le=10000, strict=True, ge=1)]] = None,
        from_name: Optional[StrictStr] = None,
    ) -> Iterator[IcebergTable]:
        """Iterate through ``IcebergTable`` objects from Snowflake, filtering on any optional 'like' pattern.

        Parameters
        __________
        like: str, optional
            A case-insensitive string functioning as a filter, with support for SQL
            wildcard characters (% and _).
        starts_with: str, optional
            String used to filter the command output based on the string of characters that appear
            at the beginning of the object name. Uses case-sensitive pattern matching.
        show_limit: int, optional
            Limit of the maximum number of rows returned by iter(). The default is ``None``, which behaves equivalently
            to show_limit=10000. This value must be between ``1`` and ``10000``.
        from_name: str, optional
            Fetch rows only following the first row whose object name matches
            the specified string. This is case-sensitive and does not have to be the full name.

        Examples
        ________
        Showing all Iceberg tables that you have access to see:

        >>> iceberg_tables = iceberg_table_collection.iter()

        Showing information of the exact Iceberg table you want to see:

        >>> iceberg_tables = iceberg_table_collection.iter(like="your-iceberg-table-name")

        Showing Iceberg tables starting with 'your-iceberg-table-name-':

        >>> iceberg_tables = iceberg_table_collection.iter(like="your-iceberg-table-name-%")

        Using a for loop to retrieve information from iterator:

        >>> for iceberg_table in iceberg_tables:
        ...     print(iceberg_table.name)
        """
        iceberg_tables = self._api.list_iceberg_tables(
            database=self.database.name,
            var_schema=self.schema.name,
            like=like,
            starts_with=starts_with,
            show_limit=show_limit,
            from_name=from_name,
            async_req=False,
        )
        return iter(iceberg_tables)

    @api_telemetry
    def iter_async(
        self,
        *,
        like: Optional[StrictStr] = None,
        starts_with: Optional[StrictStr] = None,
        show_limit: Optional[Annotated[int, Field(le=10000, strict=True, ge=1)]] = None,
        from_name: Optional[StrictStr] = None,
    ) -> PollingOperation[Iterator[IcebergTable]]:
        """An asynchronous version of :func:`iter`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self._api.list_iceberg_tables(
            database=self.database.name,
            var_schema=self.schema.name,
            like=like,
            starts_with=starts_with,
            show_limit=show_limit,
            from_name=from_name,
            async_req=True,
        )
        return PollingOperations.iterator(future)

    @staticmethod
    def _validate_iceberg_table(
        iceberg_table: Union[IcebergTable, str],
        as_select: Optional[str] = None,
        like: Optional[str] = None,
        from_aws_glue_catalog: Optional[bool] = None,
        from_delta: Optional[bool] = None,
        from_iceberg_files: Optional[bool] = None,
        from_iceberg_rest: Optional[bool] = None,
        clone_iceberg_table: Optional[Union[str, Clone]] = None,
    ) -> None:
        if not isinstance(iceberg_table, IcebergTable):
            raise TypeError("iceberg_table has to be IcebergTable object")

        options_map = {
            "as_select": as_select,
            "like": like,
            "from_aws_glue_catalog": from_aws_glue_catalog,
            "from_delta": from_delta,
            "from_iceberg_files": from_iceberg_files,
            "from_iceberg_rest": from_iceberg_rest,
            "clone_iceberg_table": clone_iceberg_table,
        }
        if len([o for o in options_map.values() if o]) > 1:
            exclusive_options = ", ".join(k for k, v in options_map.items() if v)
            msg = f"Options {exclusive_options} are mutually exclusive"
            raise ValueError(msg)

    @overload
    def _create(
        self,
        iceberg_table: IcebergTable,
        as_select: Optional[str],
        like: Optional[str],
        from_aws_glue_catalog: Optional[bool],
        from_delta: Optional[bool],
        from_iceberg_files: Optional[bool],
        from_iceberg_rest: Optional[bool],
        clone_iceberg_table: Optional[Union[str, Clone]],
        copy_grants: Optional[bool],
        mode: CreateMode,
        async_req: Literal[True],
    ) -> Future[SuccessResponse]: ...

    @overload
    def _create(
        self,
        iceberg_table: IcebergTable,
        as_select: Optional[str],
        like: Optional[str],
        from_aws_glue_catalog: Optional[bool],
        from_delta: Optional[bool],
        from_iceberg_files: Optional[bool],
        from_iceberg_rest: Optional[bool],
        clone_iceberg_table: Optional[Union[str, Clone]],
        copy_grants: Optional[bool],
        mode: CreateMode,
        async_req: Literal[False],
    ) -> SuccessResponse: ...

    def _create(
        self,
        iceberg_table: IcebergTable,
        as_select: Optional[str],
        like: Optional[str],
        from_aws_glue_catalog: Optional[bool],
        from_delta: Optional[bool],
        from_iceberg_files: Optional[bool],
        from_iceberg_rest: Optional[bool],
        clone_iceberg_table: Optional[Union[str, Clone]],
        copy_grants: Optional[bool],
        mode: CreateMode,
        async_req: bool,
    ) -> Union[SuccessResponse, Future[SuccessResponse]]:
        real_mode = CreateMode[mode].value

        self._validate_iceberg_table(
            iceberg_table=iceberg_table,
            as_select=as_select,
            like=like,
            from_aws_glue_catalog=from_aws_glue_catalog,
            from_delta=from_delta,
            from_iceberg_files=from_iceberg_files,
            from_iceberg_rest=from_iceberg_rest,
            clone_iceberg_table=clone_iceberg_table,
        )

        if as_select:
            return self._api.create_snowflake_managed_iceberg_table_as_select(
                database=self.database.name,
                var_schema=self.schema.name,
                query=as_select,
                iceberg_table_as_select=IcebergTableAsSelect(**iceberg_table.to_dict()),
                create_mode=StrictStr(real_mode),
                copy_grants=copy_grants,
                async_req=async_req,
            )
        if like:
            return self._api.create_snowflake_managed_iceberg_table_like(
                database=self.database.name,
                var_schema=self.schema.name,
                name=like,
                iceberg_table_like=IcebergTableLike(**iceberg_table.to_dict()),
                create_mode=StrictStr(real_mode),
                copy_grants=copy_grants,
                target_database=self.database.name,
                target_schema=self.schema.name,
                async_req=async_req,
            )
        if from_aws_glue_catalog:
            return self._api.create_unmanaged_iceberg_table_from_aws_glue_catalog(
                database=self.database.name,
                var_schema=self.schema.name,
                iceberg_table_from_aws_glue_catalog=IcebergTableFromAWSGlueCatalog(**iceberg_table.to_dict()),
                create_mode=StrictStr(real_mode),
                async_req=async_req,
            )
        if from_delta:
            return self._api.create_unmanaged_iceberg_table_from_delta(
                database=self.database.name,
                var_schema=self.schema.name,
                iceberg_table_from_delta=IcebergTableFromDelta(**iceberg_table.to_dict()),
                create_mode=StrictStr(real_mode),
                async_req=async_req,
            )
        if from_iceberg_files:
            return self._api.create_unmanaged_iceberg_table_from_iceberg_files(
                database=self.database.name,
                var_schema=self.schema.name,
                iceberg_table_from_iceberg_files=IcebergTableFromIcebergFiles(**iceberg_table.to_dict()),
                create_mode=StrictStr(real_mode),
                async_req=async_req,
            )
        if from_iceberg_rest:
            return self._api.create_unmanaged_iceberg_table_from_iceberg_rest(
                database=self.database.name,
                var_schema=self.schema.name,
                iceberg_table_from_iceberg_rest=IcebergTableFromIcebergRest(**iceberg_table.to_dict()),
                create_mode=StrictStr(real_mode),
                async_req=async_req,
            )
        if clone_iceberg_table:
            pot: Optional[IcebergTablePointOfTime] = None
            if isinstance(clone_iceberg_table, Clone) and isinstance(clone_iceberg_table.point_of_time, PointOfTime):
                pot = IcebergTablePointOfTime.from_dict(clone_iceberg_table.point_of_time.to_dict())

            real_clone = (
                Clone(source=clone_iceberg_table) if isinstance(clone_iceberg_table, str) else clone_iceberg_table
            )
            req = IcebergTableClone(point_of_time=pot, name=iceberg_table.name)

            source_iceberg_table_fqn = FQN.from_string(real_clone.source)
            return self._api.clone_snowflake_managed_iceberg_table(
                source_iceberg_table_fqn.database or self.database.name,
                source_iceberg_table_fqn.schema or self.schema.name,
                source_iceberg_table_fqn.name,
                iceberg_table_clone=req,
                create_mode=StrictStr(real_mode),
                async_req=async_req,
                target_database=self.database.name,
                target_schema=self.schema.name,
                copy_grants=copy_grants,
            )

        return self._api.create_snowflake_managed_iceberg_table(
            self.database.name,
            self.schema.name,
            iceberg_table,
            create_mode=StrictStr(real_mode),
            async_req=async_req,
            copy_grants=copy_grants,
        )


class IcebergTableResource(SchemaObjectReferenceMixin[IcebergTableCollection]):
    """Represents a reference to a Snowflake Iceberg table.

    With this Iceberg table reference, you can fetch information about an Iceberg table, as well as
    perform certain actions on it.
    """

    def __init__(self, name: StrictStr, collection: IcebergTableCollection) -> None:
        self.name = name
        self.collection = collection

    @api_telemetry
    def fetch(self) -> IcebergTable:
        """Fetch the details of an Iceberg table.

        Examples
        ________
        Fetching a reference to an Iceberg table to print its name:

        >>> iceberg_table_reference = root.databases["my_db"].schemas["my_schema"].iceberg_tables["foo"]
        >>> my_iceberg_table = iceberg_table_reference.fetch()
        >>> print(my_iceberg_table.name)
        """
        return self.collection._api.fetch_iceberg_table(
            self.database.name, self.schema.name, self.name, async_req=False
        )

    @api_telemetry
    def fetch_async(self) -> PollingOperation[IcebergTable]:
        """An asynchronous version of :func:`fetch`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.fetch_iceberg_table(
            self.database.name, self.schema.name, self.name, async_req=True
        )
        return PollingOperations.identity(future)

    @api_telemetry
    def drop(self, if_exists: bool = False) -> None:
        """Drop this Iceberg table.

        Parameters
        __________
        if_exists: bool, optional
            Whether to error if the Iceberg table doesn't exist. Default is ``False``.

        Examples
        ________
        Deleting an Iceberg table using its reference, error if it doesn't exist:

        >>> iceberg_table_reference.drop()

        Deleting an Iceberg table using its reference, if it exists:

        >>> iceberg_table_reference.drop(if_exists=True)
        """
        self.collection._api.drop_iceberg_table(
            self.database.name, self.schema.name, self.name, if_exists=if_exists, async_req=False
        )

    @api_telemetry
    def drop_async(self, if_exists: bool = False) -> PollingOperation[None]:
        """An asynchronous version of :func:`drop`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.drop_iceberg_table(
            self.database.name, self.schema.name, self.name, if_exists=if_exists, async_req=True
        )
        return PollingOperations.empty(future)

    @api_telemetry
    def convert_to_managed(
        self,
        base_location: Optional[StrictStr] = None,
        storage_serialization_policy: Optional[StrictStr] = None,
        if_exists: bool = False,
    ) -> None:
        """Convert Iceberg table using external Iceberg catalog into a table that uses Snowflake as the catalog.

        Parameters
        __________
        base_location: str, optional
            The path to a directory where Snowflake can write data and metadata files for the table.
        storage_serialization_policy: str, optional
            Specifies the storage serialization policy for the table.
        if_exists: bool, optional
            Whether to error if the Iceberg table doesn't exist. Default is ``False``.

        Examples
        ________
        Converts an Iceberg table to managed table using its reference, errors if it doesn't exist:

        >>> iceberg_table.convert_to_managed()

        Converts an Iceberg table to managed table using its reference, if it exists:

        >>> iceberg_table.convert_to_managed(if_exists=True)

        """
        self.collection._api.convert_to_managed_iceberg_table(
            self.database.name,
            self.schema.name,
            self.name,
            convert_to_managed_iceberg_table_request=ConvertToManagedIcebergTableRequest(
                base_location=base_location, storage_serialization_policy=storage_serialization_policy
            ),
            if_exists=if_exists,
            async_req=False,
        )

    @api_telemetry
    def convert_to_managed_async(
        self,
        base_location: Optional[StrictStr] = None,
        storage_serialization_policy: Optional[StrictStr] = None,
        if_exists: bool = False,
    ) -> PollingOperation[None]:
        """An asynchronous version of :func:`convert_to_managed`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.convert_to_managed_iceberg_table(
            self.database.name,
            self.schema.name,
            self.name,
            convert_to_managed_iceberg_table_request=ConvertToManagedIcebergTableRequest(
                base_location=base_location, storage_serialization_policy=storage_serialization_policy
            ),
            if_exists=if_exists,
            async_req=True,
        )
        return PollingOperations.empty(future)

    @api_telemetry
    def refresh(self, metadata_file_relative_path: Optional[StrictStr] = None, if_exists: bool = False) -> None:
        """Refresh the metadata for an Iceberg table that uses an external Iceberg catalog.

        Parameters
        __________
        metadata_file_relative_path: str, optional
            Specifies a metadata file path for a table created from Iceberg files in object storage.
        if_exists: bool, optional
            Whether to error if the Iceberg table doesn't exist. Default is ``False``.

        Examples
        ________
        To refresh an Iceberg table using its reference, error if it doesn't exist:

        >>> iceberg_table.refresh()

        To refresh an Iceberg table using its reference, if it exists:

        >>> iceberg_table.refresh(if_exists=True)

        """
        self.collection._api.refresh_iceberg_table(
            self.database.name,
            self.schema.name,
            self.name,
            refresh_iceberg_table_request=RefreshIcebergTableRequest(
                metadata_file_relative_path=metadata_file_relative_path
            ),
            if_exists=if_exists,
            async_req=False,
        )

    @api_telemetry
    def refresh_async(
        self, metadata_file_relative_path: Optional[StrictStr] = None, if_exists: bool = False
    ) -> PollingOperation[None]:
        """An asynchronous version of :func:`refresh`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.refresh_iceberg_table(
            self.database.name,
            self.schema.name,
            self.name,
            refresh_iceberg_table_request=RefreshIcebergTableRequest(
                metadata_file_relative_path=metadata_file_relative_path
            ),
            if_exists=if_exists,
            async_req=True,
        )
        return PollingOperations.empty(future)

    @api_telemetry
    def resume_recluster(self, if_exists: bool = False) -> None:
        """Resume reclustering for an Iceberg table.

        Parameters
        __________
        if_exists: bool, optional
            Whether to error if the Iceberg table doesn't exist. Default is ``False``.

        Examples
        ________
        To resume reclustering for an Iceberg table using its reference, error if it doesn't exist:

        >>> iceberg_table.resume_recluster()

        To resume reclustering for an Iceberg table using its reference, if it exists:

        >>> iceberg_table.resume_recluster(if_exists=True)

        """
        self.collection._api.resume_recluster_iceberg_table(
            self.database.name, self.schema.name, self.name, if_exists=if_exists, async_req=False
        )

    @api_telemetry
    def resume_recluster_async(self, if_exists: bool = False) -> PollingOperation[None]:
        """An asynchronous version of :func:`resume_recluster`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.resume_recluster_iceberg_table(
            self.database.name, self.schema.name, self.name, if_exists=if_exists, async_req=True
        )
        return PollingOperations.empty(future)

    @api_telemetry
    def suspend_recluster(self, if_exists: bool = False) -> None:
        """Suspend reclustering for an Iceberg table.

        Parameters
        __________
        if_exists: bool, optional
            Whether to error if the Iceberg table doesn't exist. Default is ``False``.

        Examples
        ________
        To suspend reclustering for an Iceberg table using its reference, error if it doesn't exist:

        >>> iceberg_table.suspend_recluster()

        To suspend reclustering for an Iceberg table using its reference, if it exists:

        >>> iceberg_table.suspend_recluster(if_exists=True)

        """
        self.collection._api.suspend_recluster_iceberg_table(
            self.database.name, self.schema.name, self.name, if_exists=if_exists, async_req=False
        )

    @api_telemetry
    def suspend_recluster_async(self, if_exists: bool = False) -> PollingOperation[None]:
        """An asynchronous version of :func:`suspend_recluster`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.suspend_recluster_iceberg_table(
            self.database.name, self.schema.name, self.name, if_exists=if_exists, async_req=True
        )
        return PollingOperations.empty(future)

    @api_telemetry
    def undrop(self) -> None:
        """Undrop an Iceberg table.

        Parameters
        __________
        if_exists: bool, optional
            Whether to error if the Iceberg table doesn't exist. Default is ``False``.

        Examples
        ________
        To undrop an Iceberg table using its reference, error if it doesn't exist:

        >>> iceberg_table.undrop()

        To undrop an Iceberg table using its reference, if it exists:

        >>> iceberg_table.undrop(if_exists=True)

        """
        self.collection._api.undrop_iceberg_table(self.database.name, self.schema.name, self.name, async_req=False)

    @api_telemetry
    def undrop_async(self) -> PollingOperation[None]:
        """An asynchronous version of :func:`undrop`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.undrop_iceberg_table(
            self.database.name, self.schema.name, self.name, async_req=True
        )
        return PollingOperations.empty(future)

    @api_telemetry
    def set_tags(self, tags: dict[TagResource, TagValue], if_exists: Optional[bool] = None) -> None:
        """Set tags on this Iceberg table.

        Parameters
        __________
        if_exists: bool, optional
            Check the existence of this Iceberg table before setting the tags.
            Default is ``None``, which is equivalent to ``False``.

        Examples
        ________
        Set tags on an Iceberg table:

        >>> tag = root.databases["your-database"].schemas["your-schema"].tags["your-tag"]
        ... other_tag = root.databases["your-database"].schemas["your-schema"].tags["your-other-tag"]
        ... iceberg_table_reference.set_tags({tag: TagValue("your-value"), other_tag: TagValue("your-value")})
        """
        self.collection._api.set_tags(
            database=self.database.name,
            var_schema=self.schema.name,
            name=self.name,
            tag_assignment=[
                tag_tuple_to_tag_assignment(TagAssignment, tag_resource, tag_value)
                for [tag_resource, tag_value] in tags.items()
            ],
            if_exists=if_exists,
        )

    @api_telemetry
    def set_tags_async(
        self, tags: dict[TagResource, TagValue], if_exists: Optional[bool] = None
    ) -> PollingOperation[None]:
        """An asynchronous version of :func:`set_tags`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.set_tags(
            database=self.database.name,
            var_schema=self.schema.name,
            name=self.name,
            tag_assignment=[
                tag_tuple_to_tag_assignment(TagAssignment, tag_resource, tag_value)
                for [tag_resource, tag_value] in tags.items()
            ],
            if_exists=if_exists,
            async_req=True,
        )
        return PollingOperations.empty(future)

    @api_telemetry
    def unset_tags(self, tag_resources: set[TagResource], if_exists: Optional[bool] = None) -> None:
        """Unset tags from this Iceberg table.

        Parameters
        __________
        if_exists: bool, optional
            Check the existence of this Iceberg table before unsetting the tags.
            Default is ``None``, which is equivalent to ``False``.

        Examples
        ________
        Unset tags from an Iceberg table:

        >>> tag = root.databases["your-database"].schemas["your-schema"].tags["your-tag"]
        ... other_tag = root.databases["your-database"].schemas["your-schema"].tags["your-other-tag"]
        ... iceberg_table_reference.unset_tags({tag, other_tag})
        """
        self.collection._api.unset_tags(
            database=self.database.name,
            var_schema=self.schema.name,
            name=self.name,
            tag_reference=[tag_resource_to_tag_reference(TagReference, tag_resource) for tag_resource in tag_resources],
            if_exists=if_exists,
        )

    @api_telemetry
    def unset_tags_async(
        self, tag_resources: set[TagResource], if_exists: Optional[bool] = None
    ) -> PollingOperation[None]:
        """An asynchronous version of :func:`unset_tags`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.unset_tags(
            database=self.database.name,
            var_schema=self.schema.name,
            name=self.name,
            tag_reference=[tag_resource_to_tag_reference(TagReference, tag_resource) for tag_resource in tag_resources],
            if_exists=if_exists,
            async_req=True,
        )
        return PollingOperations.empty(future)

    @api_telemetry
    def get_tags(self, with_lineage: Optional[bool] = None) -> dict[TagResource, TagValue]:
        """Fetch all tags assigned to this Iceberg table.

        This operation requires an active warehouse.

        Parameters
        __________
        with_lineage: bool, optional
            Controls whether returned tag assignments include only tags directly set on the object (False)
            or also those inherited through Snowflake’s object hierarchy (True).
            Default is ``None``, which is equivalent to ``False``.

        Examples
        ________
        Get tags set on an Iceberg table:

        >>> iceberg_table_reference.get_tags()
        """
        tag_assignments = self.collection._api.get_tags(
            database=self.database.name,
            var_schema=self.schema.name,
            name=self.name,
            with_lineage=with_lineage,
        )
        return dict(tag_assignment_to_tag_tuple(ta, self.root) for ta in tag_assignments)

    @api_telemetry
    def get_tags_async(self, with_lineage: Optional[bool] = None) -> PollingOperation[dict[TagResource, TagValue]]:
        """An asynchronous version of :func:`get_tags`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.get_tags(
            database=self.database.name,
            var_schema=self.schema.name,
            name=self.name,
            with_lineage=with_lineage,
            async_req=True,
        )
        return PollingOperation(
            future, lambda tag_assignments: dict(tag_assignment_to_tag_tuple(ta, self.root) for ta in tag_assignments)
        )



from contextlib import suppress

import pytest

from snowflake.core.exceptions import NotFoundError
from snowflake.core.streamlit import (
    AddLiveVersionStreamlitRequest,
    CommitStreamlitRequest,
    Streamlit,
)
from tests.integ.utils import random_string


@pytest.mark.min_sf_ver("9.38.0")
def test_live_and_commit(streamlits, streamlit_stage_with_file, warehouse, streamlit_main_file):
    name = random_string(8, "test_streamlit_")
    st = Streamlit(
        name=name,
        query_warehouse=warehouse.name,
        source_location=f"@{streamlit_stage_with_file.name}",
        main_file=streamlit_main_file,
    )

    ref = streamlits.create(st)
    try:
        assert ref.fetch().live_version_location_uri is None
        ref.add_live_version(from_last=True, add_live_version_streamlit_request=AddLiveVersionStreamlitRequest())
        assert ref.fetch().live_version_location_uri is not None
        ref.commit(commit_streamlit_request=CommitStreamlitRequest())
        assert ref.fetch().live_version_location_uri is None
    finally:
        with suppress(NotFoundError):
            ref.drop()



# mypy: ignore-errors
import warnings

from concurrent.futures import Future
from typing import TYPE_CHECKING, Any, Literal, Optional, Union, overload

from pydantic import StrictStr

from snowflake.core import PollingOperation

from .._internal.telemetry import api_telemetry
from .._utils import map_result
from ._generated.api.procedure_api_base import ProcedureCollectionBase, ProcedureResourceBase
from ._generated.models.call_argument_list import CallArgumentList


if TYPE_CHECKING:
    from snowflake.core.schema import SchemaResource


class ProcedureCollection(ProcedureCollectionBase):
    """Represents the collection operations on the Snowflake Procedure resource.

    With this collection, you can create, iterate through, and fetch procedures that you have access to
    in the current context.

    Examples
    ________
    Creating a procedure instance:

    >>> procedure = Procedure(
    ...     name="sql_proc_table_func",
    ...     arguments=[Argument(name="id", datatype="VARCHAR")],
    ...     return_type=ReturnTable(
    ...         column_list=[
    ...             ColumnType(name="id", datatype="NUMBER"),
    ...             ColumnType(name="price", datatype="NUMBER"),
    ...         ]
    ...     ),
    ...     language_config=SQLFunction(),
    ...     body="
    ...         DECLARE
    ...             res RESULTSET DEFAULT (SELECT * FROM invoices WHERE id = :id);
    ...         BEGIN
    ...             RETURN TABLE(res);
    ...         END;
    ...     ",
    ... )
    >>> procedures = root.databases["my_db"].schemas["my_schema"].procedures
    >>> procedures.create(procedure)
    """

    _identifier_requires_args = True

    def __init__(self, schema: "SchemaResource") -> None:
        super().__init__(schema, ProcedureResource)


class ProcedureResource(ProcedureResourceBase):
    """Represents a reference to a Snowflake procedure.

    With this procedure reference, you can fetch information about a procedure, as well as
    perform certain actions on it.
    """

    _identifier_requires_args = True
    _plural_name = "procedures"

    def __init__(self, name_with_args: StrictStr, collection_class: ProcedureCollection) -> None:
        super().__init__(name_with_args, collection_class)

    @api_telemetry
    def call(self, call_argument_list: Optional[CallArgumentList] = None, extract: Optional[bool] = False) -> Any:
        """Call this procedure.

        Examples
        ________
        Calling a procedure with no arguments using its reference:

        >>> procedure_reference.call(call_argument_list=CallArgumentList(call_arguments=[]))

        Calling a procedure with 2 arguments using its reference:

        >>> procedure_reference.call(
        ...     call_argument_list=CallArgumentList(
        ...         call_arguments=[
        ...             CallArgument(name="id", datatype="NUMBER", value=1),
        ...             CallArgument(name="tableName", datatype="VARCHAR", value="my_table_name"),
        ...         ]
        ...     )
        ... )
        """
        return self._call(call_argument_list=call_argument_list, async_req=False, extract=extract)

    @api_telemetry
    def call_async(
        self, call_argument_list: Optional[CallArgumentList] = None, extract: Optional[bool] = False
    ) -> PollingOperation[Any]:
        """An asynchronous version of :func:`call`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        return self._call(call_argument_list=call_argument_list, async_req=True, extract=extract)

    @overload
    def _call(
        self, call_argument_list: Optional[CallArgumentList], extract: Optional[bool], async_req: Literal[True]
    ) -> PollingOperation[Any]: ...

    @overload
    def _call(
        self, call_argument_list: Optional[CallArgumentList], extract: Optional[bool], async_req: Literal[False]
    ) -> Any: ...

    def _call(
        self, call_argument_list: Optional[CallArgumentList], async_req: bool, extract: bool = False
    ) -> Union[Any, PollingOperation[Any]]:
        if extract is False:
            warnings.warn(
                "Please use `extract=True` when calling procedure. This will extract "
                "result from [{sproc_name: result}] object. This will become default behavior.",
                DeprecationWarning,
                stacklevel=4,
            )

        # None is not supported by self.collection._api.call_procedure
        if call_argument_list is None:
            call_argument_list = CallArgumentList(call_arguments=[])

        procedure = self.fetch()
        for argument in procedure.arguments:
            if argument.default_value is None:
                assert any(
                    argument.name.upper() == call_argument.name.upper()
                    for call_argument in call_argument_list.call_arguments
                )

        result_or_future = self.collection._api.call_procedure(
            self.database.name,
            self.schema.name,
            procedure.name,
            call_argument_list=call_argument_list,
            async_req=async_req,
        )

        def mapper(r):
            return map_result(procedure, r, extract=extract)

        if isinstance(result_or_future, Future):
            return PollingOperation(result_or_future, mapper)
        return mapper(result_or_future)



from contextlib import suppress

import pytest

from tests.integ.utils import (
    backup_database_and_schema,
    setup_credentials,
    setup_spcs,
    should_disable_setup_for_credentials,
    should_disable_setup_for_spcs,
)

from ...utils import is_prod_or_preprod
from .constants import (
    TEST_COMPUTE_POOL,
    TEST_SHARED_TAG_DATABASE,
    TEST_SHARED_TAG_SCHEMA,
    TEST_WAREHOUSE,
    SpcsSetupTuple,
    objects_to_setup,
)


# Setup Warehouse
@pytest.fixture(scope="session")
def warehouse_setup(cursor):
    cursor.execute(f"CREATE WAREHOUSE IF NOT EXISTS {TEST_WAREHOUSE};").fetchone()
    cursor.execute(f"USE WAREHOUSE {TEST_WAREHOUSE};").fetchone()


@pytest.fixture(scope="session")
def shared_tag_setup(cursor):
    with backup_database_and_schema(cursor):
        cursor.execute(f"CREATE DATABASE IF NOT EXISTS {TEST_SHARED_TAG_DATABASE}")
        cursor.execute(f"CREATE SCHEMA IF NOT EXISTS {TEST_SHARED_TAG_SCHEMA}")


# Setup basic objects: database, schema
@pytest.fixture(scope="session", autouse=True)
def setup_basic(connection):
    with connection.cursor() as cursor, backup_database_and_schema(cursor):
        _temp_database = None
        _temp_schema = None

        for db_name, db in objects_to_setup.items():
            cursor.execute(f"CREATE DATABASE IF NOT EXISTS /* setup_basic */ {db_name} {db['params']}")
            _temp_database = db_name

            cursor.execute(f"USE DATABASE /* setup_basic */ {db_name}")  # just in case it already existed
            for schema_name in db["schemas"]:
                cursor.execute(f"CREATE SCHEMA IF NOT EXISTS /* setup_basic */ {schema_name}")
                _temp_schema = schema_name

        if _temp_database is not None:
            cursor.execute(f"USE DATABASE /* setup_basic */ {_temp_database}")
        if _temp_schema is not None:
            cursor.execute(f"USE SCHEMA /* setup_basic */ {_temp_schema}")

        try:
            yield
        finally:
            if _temp_database is not None:
                cursor.execute(f"DROP DATABASE /* setup_basic::reset */ {_temp_database}")


# Setup Compute pool by either create FAKE instance family or use CPU_X64_XS
@pytest.fixture(scope="session")
def spcs_setup_objects(cursor, sf_cursor, db_parameters, test_account, snowflake_version, snowflake_region):
    is_prod_or_preprod_env = is_prod_or_preprod(snowflake_version, snowflake_region)
    if not should_disable_setup_for_spcs(db_parameters) and (not is_prod_or_preprod_env):
        if sf_cursor is None:
            pytest.fail(
                'Either disable setup for SPCS by setting "should_disable_setup_for_spcs" to "true" in '
                "connection config for account you are running the test on or provide a connection "
                "configuration named sf_account = {account=..., database = ....}"
            )
        else:
            setup_spcs(
                executing_account_cursor=sf_cursor,
                target_account_name=test_account,
                instance_families_to_create=["CPU_X64_XS", "FAKE"],
            )
    if is_prod_or_preprod_env:
        instance_family = "CPU_X64_XS"
    else:
        instance_family = "FAKE"

    cursor.execute(
        f"create compute pool if not exists {TEST_COMPUTE_POOL} "
        + f"with instance_family={instance_family} "
        + "min_nodes=1 max_nodes=5 auto_resume=true auto_suspend_secs=60;"
    ).fetchone()[0]

    with suppress(Exception):
        yield SpcsSetupTuple(instance_family, TEST_COMPUTE_POOL)


@pytest.fixture()
def setup_credentials_fixture(sf_cursor, db_parameters, snowflake_version, snowflake_region):
    is_prod_or_preprod_env = is_prod_or_preprod(snowflake_version, snowflake_region)
    if not should_disable_setup_for_credentials(db_parameters) and (not is_prod_or_preprod_env):
        if sf_cursor is None:
            pytest.fail(
                'Either disable setup for credentials by setting "should_disable_setup_for_credentials" to "true" in '
                "connection config for account you are running the test on or provide a connection "
                "configuration named sf_account = {account=..., database = ....}"
            )
        else:
            setup_credentials(sf_cursor)



from concurrent.futures.thread import ThreadPoolExecutor
from threading import Lock
from typing import Optional


class _SharedThreadPool:
    def __init__(self) -> None:
        self._pool: Optional[ThreadPoolExecutor] = None
        self._lock = Lock()

    def get(self, max_threads: Optional[int] = None) -> ThreadPoolExecutor:
        with self._lock:
            if self._pool is None:
                self._pool = ThreadPoolExecutor(max_workers=max_threads, thread_name_prefix="api-client-worker")
            return self._pool

    def reset(self) -> None:
        with self._lock:
            if self._pool is not None:
                self._pool.shutdown()
            self._pool = None


# Use a thread pool singleton for every resource
THREAD_POOL = _SharedThreadPool()


def get_thread_pool(max_threads: Optional[int] = None) -> ThreadPoolExecutor:
    return THREAD_POOL.get(max_threads=max_threads)



from typing import Optional

import pytest

from snowflake.core import CreateMode
from snowflake.core.catalog_integration import CatalogIntegration, CatalogIntegrationCollection, ObjectStore
from snowflake.core.tag import TagResource, TagValue

from ..base_tag_tests import BaseTagTests
from ..utils import random_string


pytestmark = [
    pytest.mark.internal_only,
    pytest.mark.usefixtures("setup_credentials_fixture"),
]


class TestCatalogIntegrationTags(BaseTagTests):
    @pytest.fixture(autouse=True)
    def setup(self, catalog_integrations: CatalogIntegrationCollection):
        self.catalog_integrations = catalog_integrations
        integration_name = random_string(6, "catalog_integration_for_tag_tests_")
        integration = CatalogIntegration(
            name=integration_name,
            catalog=ObjectStore(),
            table_format="ICEBERG",
            enabled=True,
        )
        self.integration_res = catalog_integrations.create(integration, mode=CreateMode.if_not_exists)
        try:
            yield
        finally:
            self.integration_res.drop(if_exists=True)

    @property
    def resource_level_name(self) -> str:
        return "INTEGRATION"

    def set_tags(
        self, tags: dict[TagResource, TagValue], if_exists: Optional[bool] = False, resource_name: Optional[str] = None
    ):
        self.catalog_integrations[resource_name or self.integration_res.name].set_tags(tags, if_exists)

    def unset_tags(
        self, tag_resources: set[TagResource], if_exists: Optional[bool] = False, resource_name: Optional[str] = None
    ):
        self.catalog_integrations[resource_name or self.integration_res.name].unset_tags(tag_resources, if_exists)

    def get_tags(
        self, with_lineage: Optional[bool] = False, resource_name: Optional[str] = None
    ) -> dict[TagResource, TagValue]:
        return self.catalog_integrations[resource_name or self.integration_res.name].get_tags(with_lineage)



from unittest import mock

import pytest

from snowflake.core import PollingOperation
from snowflake.core.stream import Stream, StreamResource, StreamSourceTable
from snowflake.core.stream._generated import TagAssignment, TagReference
from snowflake.core.tag import TagValue

from ...utils import BASE_URL, extra_params, mock_http_response


API_CLIENT_REQUEST = "snowflake.core._generated.api_client.ApiClient.request"
STREAM = Stream(name="my_stream", stream_source=StreamSourceTable(name="my_tab"))


@pytest.fixture
def streams(schema):
    return schema.streams


@pytest.fixture
def stream(streams):
    return streams["my_stream"]


def test_create_stream(fake_root, streams):
    args = (
        fake_root,
        "POST",
        BASE_URL + "/databases/my_db/schemas/my_schema/streams?createMode=errorIfExists&copyGrants=False",
    )
    kwargs = extra_params(
        query_params=[("createMode", "errorIfExists"), ("copyGrants", False)],
        body={"name": "my_stream", "stream_source": {"name": "my_tab", "src_type": "table"}},
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        stream_res = streams.create(STREAM)
        assert isinstance(stream_res, StreamResource)
        assert stream_res.name == "my_stream"
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = streams.create_async(STREAM)
        assert isinstance(op, PollingOperation)
        stream_res = op.result()
        assert stream_res.name == "my_stream"
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_create_stream_clone(fake_root, streams):
    args = (
        fake_root,
        "POST",
        BASE_URL
        + "/databases/my_db/schemas/my_schema/streams/clone_stream:clone?"
        + "createMode=errorIfExists&targetDatabase=my_db&targetSchema=my_schema&copyGrants=False",
    )
    kwargs = extra_params(
        query_params=[
            ("createMode", "errorIfExists"),
            ("targetDatabase", streams.database.name),
            ("targetSchema", streams.schema.name),
            ("copyGrants", False),
        ],
        body={"name": "my_stream"},
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        streams.create("my_stream", clone_stream="clone_stream")
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = streams.create_async("my_stream", clone_stream="clone_stream")
        assert isinstance(op, PollingOperation)
        stream_res = op.result()
        assert stream_res.name == "my_stream"
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_iter_stream(fake_root, streams):
    args = (fake_root, "GET", BASE_URL + "/databases/my_db/schemas/my_schema/streams")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        streams.iter()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        op = streams.iter_async()
        assert isinstance(op, PollingOperation)
        it = op.result()
        assert list(it) == []
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_iter_show_limit_deprecation_warning(fake_root, streams):
    """Test that show_limit parameter triggers deprecation warning."""
    args = (
        fake_root,
        "GET",
        BASE_URL + "/databases/my_db/schemas/my_schema/streams?showLimit=10",
    )
    kwargs = extra_params(query_params=[("showLimit", 10)])

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        with pytest.warns(DeprecationWarning, match="'show_limit' is deprecated, use 'limit' instead"):
            list(streams.iter(show_limit=10))
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        with pytest.warns(DeprecationWarning, match="'show_limit' is deprecated, use 'limit' instead"):
            streams.iter_async(show_limit=10).result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_iter_show_limit(fake_root, streams):
    args = (
        fake_root,
        "GET",
        BASE_URL + "/databases/my_db/schemas/my_schema/streams?showLimit=10",
    )
    kwargs = extra_params(query_params=[("showLimit", 10)])

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        list(streams.iter(limit=10))
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        streams.iter_async(limit=10).result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_iter_limit_and_show_limit_conflict(streams):
    """Test that providing both limit and show_limit raises ValueError."""
    with pytest.raises(ValueError, match="Cannot specify both 'limit' and 'show_limit'"):
        list(streams.iter(limit=10, show_limit=5))

    with pytest.raises(ValueError, match="Cannot specify both 'limit' and 'show_limit'"):
        streams.iter_async(limit=10, show_limit=5).result()


def test_fetch_stream(fake_root, stream):
    from snowflake.core.stream._generated.models import Stream as StreamModel
    from snowflake.core.stream._generated.models import StreamSourceTable as StreamSourceTableModel

    model = StreamModel(name="my_stream", stream_source=StreamSourceTableModel(name="my_tab"))
    args = (fake_root, "GET", BASE_URL + "/databases/my_db/schemas/my_schema/streams/my_stream")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response(model.to_json())
        stream.fetch()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response(model.to_json())
        op = stream.fetch_async()
        assert isinstance(op, PollingOperation)
        stream = op.result()
        assert stream.to_dict() == STREAM.to_dict()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_drop_stream(fake_root, stream):
    args = (fake_root, "DELETE", BASE_URL + "/databases/my_db/schemas/my_schema/streams/my_stream")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        stream.drop()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = stream.drop_async()
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_set_tags(fake_root, stream, tag):
    args = (fake_root, "POST", BASE_URL + "/databases/my_db/schemas/my_schema/streams/my_stream:set-tags")
    tags = {tag: TagValue(value="value")}
    kwargs = extra_params(
        body=[
            TagAssignment(
                tag_value=v.value, tag_name=k.name, tag_schema=k.schema.name, tag_database=k.database.name
            ).to_dict()
            for k, v in tags.items()
        ]
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        stream.set_tags(tags)
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = stream.set_tags_async(tags)
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_unset_tags(fake_root, stream, tag):
    args = (fake_root, "POST", BASE_URL + "/databases/my_db/schemas/my_schema/streams/my_stream:unset-tags")
    tag_resources = {tag}
    kwargs = extra_params(
        body=[
            TagReference(
                tag_name=tag_res.name, tag_schema=tag_res.schema.name, tag_database=tag_res.database.name
            ).to_dict()
            for tag_res in tag_resources
        ]
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        stream.unset_tags(tag_resources)
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = stream.unset_tags_async(tag_resources)
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_get_tags(fake_root, stream):
    args = (fake_root, "GET", BASE_URL + "/databases/my_db/schemas/my_schema/streams/my_stream:get-tags")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        assert stream.get_tags() == {}
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        op = stream.get_tags_async()
        assert isinstance(op, PollingOperation)
        assert op.result() == {}
    mocked_request.assert_called_once_with(*args, **kwargs)



from __future__ import absolute_import

# flake8: noqa

# import apis into api package
from snowflake.core.cortex.lite_agent_service._generated.api.cortex_lite_agent_api import CortexLiteAgentApi

__all__ = [
    "CortexLiteAgentApi",
]



"""
Snowflake Services API.

The Snowflake Services API is a REST API that you can use to access, update, and perform certain actions on Services resource in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re

from typing import Any, List, Optional

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictStr, field_validator
from typing_extensions import Annotated

from snowflake.core.service._generated.models.service_spec import ServiceSpec, ServiceSpecModel


class JobService(BaseModel):
    """A model object representing the JobService resource.

    Constructs an object of type JobService with the provided properties.

    Parameters
    __________
    name : str
        A Snowflake object identifier. If the identifier contains spaces or special characters, the entire string must be enclosed in double quotes. Identifiers enclosed in double quotes are also case-sensitive.
    compute_pool : str
        Specifies the name of the compute pool in your account on which to run the service.
    spec : ServiceSpec

    status : str, optional
        The current status of the service.
    external_access_integrations : list[str], optional
        Specifies the names of the external access integrations that allow your service to access external sites.
    query_warehouse : str, optional
        A Snowflake object identifier. If the identifier contains spaces or special characters, the entire string must be enclosed in double quotes. Identifiers enclosed in double quotes are also case-sensitive.
    comment : str, optional
        Specifies a comment for the service.
    is_async_job : bool, optional
        True if the service is an async job service; false otherwise.
    """

    name: Annotated[str, Field(strict=True)]

    status: Optional[StrictStr] = None

    compute_pool: StrictStr

    spec: ServiceSpec

    external_access_integrations: Optional[List[StrictStr]] = None

    query_warehouse: Optional[Annotated[str, Field(strict=True)]] = None

    comment: Optional[StrictStr] = None

    is_async_job: Optional[StrictBool] = None

    __properties = [
        "name",
        "status",
        "compute_pool",
        "spec",
        "external_access_integrations",
        "query_warehouse",
        "comment",
        "is_async_job",
    ]

    @field_validator("name")
    def name_validate_regular_expression(cls, v):
        if not re.match(r"""^\"([^\"]|\"\")+\"|[a-zA-Z_][a-zA-Z0-9_$]*$""", v):
            raise ValueError(r"""must validate the regular expression /^"([^"]|"")+"|[a-zA-Z_][a-zA-Z0-9_$]*$/""")
        return v

    @field_validator("query_warehouse")
    def query_warehouse_validate_regular_expression(cls, v):
        if v is None:
            return v
        if not re.match(r"""^\"([^\"]|\"\")+\"|[a-zA-Z_][a-zA-Z0-9_$]*$""", v):
            raise ValueError(r"""must validate the regular expression /^"([^"]|"")+"|[a-zA-Z_][a-zA-Z0-9_$]*$/""")
        return v

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> JobService:
        """Create an instance of JobService from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        # override the default output from pydantic by calling `to_dict()` of spec
        if self.spec:
            _dict["spec"] = self.spec.to_dict()

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> JobService:
        """Create an instance of JobService from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return JobService.model_validate(obj)

        _obj = JobService.model_validate(
            {
                "name": obj.get("name"),
                "status": obj.get("status"),
                "compute_pool": obj.get("compute_pool"),
                "spec": ServiceSpec.from_dict(obj.get("spec")) if obj.get("spec") is not None else None,
                "external_access_integrations": obj.get("external_access_integrations"),
                "query_warehouse": obj.get("query_warehouse"),
                "comment": obj.get("comment"),
                "is_async_job": obj.get("is_async_job"),
            }
        )

        return _obj


class JobServiceModel:
    def __init__(
        self,
        name: str,
        compute_pool: str,
        spec: ServiceSpec,
        # optional properties
        status: Optional[str] = None,
        external_access_integrations: Optional[list[str]] = None,
        query_warehouse: Optional[str] = None,
        comment: Optional[str] = None,
        is_async_job: Optional[bool] = None,
    ):
        """A model object representing the JobService resource.

        Constructs an object of type JobService with the provided properties.

        Parameters
        __________
        name : str
            A Snowflake object identifier. If the identifier contains spaces or special characters, the entire string must be enclosed in double quotes. Identifiers enclosed in double quotes are also case-sensitive.
        compute_pool : str
            Specifies the name of the compute pool in your account on which to run the service.
        spec : ServiceSpec

        status : str, optional
            The current status of the service.
        external_access_integrations : list[str], optional
            Specifies the names of the external access integrations that allow your service to access external sites.
        query_warehouse : str, optional
            A Snowflake object identifier. If the identifier contains spaces or special characters, the entire string must be enclosed in double quotes. Identifiers enclosed in double quotes are also case-sensitive.
        comment : str, optional
            Specifies a comment for the service.
        is_async_job : bool, optional
            True if the service is an async job service; false otherwise.
        """
        self.name = name
        self.status = status
        self.compute_pool = compute_pool
        self.spec = spec
        self.external_access_integrations = external_access_integrations
        self.query_warehouse = query_warehouse
        self.comment = comment
        self.is_async_job = is_async_job

    __properties = [
        "name",
        "status",
        "compute_pool",
        "spec",
        "external_access_integrations",
        "query_warehouse",
        "comment",
        "is_async_job",
    ]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return JobService(
            name=self.name,
            status=self.status,
            compute_pool=self.compute_pool,
            spec=self.spec._to_model() if self.spec is not None else None,
            external_access_integrations=self.external_access_integrations,
            query_warehouse=self.query_warehouse,
            comment=self.comment,
            is_async_job=self.is_async_job,
        )

    @classmethod
    def _from_model(cls, model) -> JobServiceModel:
        return JobServiceModel(
            name=model.name,
            status=model.status,
            compute_pool=model.compute_pool,
            spec=ServiceSpecModel._from_model(model.spec) if model.spec else None,
            external_access_integrations=model.external_access_integrations,
            query_warehouse=model.query_warehouse,
            comment=model.comment,
            is_async_job=model.is_async_job,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a JobService.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the JobService object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> JobServiceModel:
        """Create an instance of JobService from a dict.

        This method constructs a JobService object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        JobService
            A JobService object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(JobService.from_dict(obj))


JobService._model_class = JobServiceModel



# coding: utf-8

# flake8: noqa

"""
Snowflake Streamlit API.

The Snowflake Streamlit API is a REST API that you can use to access, update, and perform certain actions on Streamlit resources in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import absolute_import

__version__ = "1.0.0"

# import apis into sdk package
from snowflake.core.streamlit._generated.api.streamlit_api import StreamlitApi

# import models into sdk package
from snowflake.core.streamlit._generated.models.add_live_version_streamlit_request import AddLiveVersionStreamlitRequest
from snowflake.core.streamlit._generated.models.add_live_version_streamlit_request_version import (
    AddLiveVersionStreamlitRequestVersion,
)
from snowflake.core.streamlit._generated.models.add_version_from_git_streamlit_request import (
    AddVersionFromGitStreamlitRequest,
)
from snowflake.core.streamlit._generated.models.add_version_streamlit_request import AddVersionStreamlitRequest
from snowflake.core.streamlit._generated.models.add_version_streamlit_request_version import (
    AddVersionStreamlitRequestVersion,
)
from snowflake.core.streamlit._generated.models.commit_streamlit_request import CommitStreamlitRequest
from snowflake.core.streamlit._generated.models.commit_streamlit_request_version import CommitStreamlitRequestVersion
from snowflake.core.streamlit._generated.models.error_response import ErrorResponse
from snowflake.core.streamlit._generated.models.git_push_with_credentials import GitPushWithCredentials
from snowflake.core.streamlit._generated.models.git_push_with_username_password import GitPushWithUsernamePassword
from snowflake.core.streamlit._generated.models.streamlit import Streamlit
from snowflake.core.streamlit._generated.models.streamlit_push_options import StreamlitPushOptions
from snowflake.core.streamlit._generated.models.streamlit_version_for_git import StreamlitVersionForGit
from snowflake.core.streamlit._generated.models.success_accepted_response import SuccessAcceptedResponse
from snowflake.core.streamlit._generated.models.success_response import SuccessResponse
from snowflake.core.streamlit._generated.models.tag_assignment import TagAssignment
from snowflake.core.streamlit._generated.models.tag_reference import TagReference
from snowflake.core.streamlit._generated.models.version_details_streamlit import VersionDetailsStreamlit



"""Manages Snowflake api integration."""

from ..api_integration._generated.models import ApiHook, ApiIntegration, AwsHook, AzureHook, GitHook, GoogleCloudHook
from ._api_integration import ApiIntegrationCollection, ApiIntegrationResource


__all__ = [
    "ApiIntegrationResource",
    "ApiIntegrationCollection",
    "ApiHook",
    "ApiIntegration",
    "AwsHook",
    "AzureHook",
    "GitHook",
    "GoogleCloudHook",
]



from unittest import mock

import pytest

from snowflake.core import PollingOperation
from snowflake.core.service import JobService, Service, ServiceResource, ServiceSpecInlineText
from snowflake.core.service._generated import FetchServiceLogs200Response, FetchServiceStatus200Response

from ...utils import BASE_URL, extra_params, mock_http_response


API_CLIENT_REQUEST = "snowflake.core._generated.api_client.ApiClient.request"
SERVICE = Service(name="my_service", compute_pool="my_compute_pool", spec=ServiceSpecInlineText(spec_text=""))
JOB_SERVICE = JobService(name="my_service", compute_pool="my_compute_pool", spec=ServiceSpecInlineText(spec_text=""))


@pytest.fixture
def services(schema):
    return schema.services


@pytest.fixture
def service(services):
    return services["my_service"]


def test_create_service(fake_root, services):
    args = (fake_root, "POST", BASE_URL + "/databases/my_db/schemas/my_schema/services?createMode=errorIfExists")
    kwargs = extra_params(
        query_params=[("createMode", "errorIfExists")],
        body={
            "name": "my_service",
            "compute_pool": "my_compute_pool",
            "spec": {"spec_text": "", "spec_type": "from_inline"},
        },
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        service_res = services.create(SERVICE)
        assert isinstance(service_res, ServiceResource)
        assert service_res.name == "my_service"
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = services.create_async(SERVICE)
        assert isinstance(op, PollingOperation)
        service_res = op.result()
        assert service_res.name == "my_service"
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_iter_service(fake_root, services):
    args = (fake_root, "GET", BASE_URL + "/databases/my_db/schemas/my_schema/services")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        services.iter()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        op = services.iter_async()
        assert isinstance(op, PollingOperation)
        it = op.result()
        assert list(it) == []
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_execute_job_service(fake_root, services):
    args = (fake_root, "POST", BASE_URL + "/databases/my_db/schemas/my_schema/services:execute-job")
    kwargs = extra_params(
        body={
            "name": "my_service",
            "compute_pool": "my_compute_pool",
            "spec": {"spec_text": "", "spec_type": "from_inline"},
        }
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        services.execute_job(JOB_SERVICE)
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = services.execute_job_async(JOB_SERVICE)
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_create_or_alter_service(fake_root, service):
    args = (fake_root, "PUT", BASE_URL + "/databases/my_db/schemas/my_schema/services/my_service")
    kwargs = extra_params(
        body={
            "name": "my_service",
            "compute_pool": "my_compute_pool",
            "spec": {"spec_text": "", "spec_type": "from_inline"},
        }
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        service.create_or_alter(SERVICE)
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = service.create_or_alter_async(SERVICE)
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_fetch_service(fake_root, service):
    from snowflake.core.service._generated.models import Service as ServiceModel
    from snowflake.core.service._generated.models import ServiceSpecInlineText as ServiceSpecInlineTextModel

    model = ServiceModel(
        name="my_service", compute_pool="my_compute_pool", spec=ServiceSpecInlineTextModel(spec_text="")
    )
    args = (fake_root, "GET", BASE_URL + "/databases/my_db/schemas/my_schema/services/my_service")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response(model.to_json())
        service.fetch()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response(model.to_json())
        op = service.fetch_async()
        assert isinstance(op, PollingOperation)
        service = op.result()
        assert service.to_dict() == SERVICE.to_dict()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_drop_service(fake_root, service):
    args = (fake_root, "DELETE", BASE_URL + "/databases/my_db/schemas/my_schema/services/my_service")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        service.drop()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = service.drop_async()
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_suspend_service(fake_root, service):
    args = (fake_root, "POST", BASE_URL + "/databases/my_db/schemas/my_schema/services/my_service:suspend")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        service.suspend()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = service.suspend_async()
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_resume_service(fake_root, service):
    args = (fake_root, "POST", BASE_URL + "/databases/my_db/schemas/my_schema/services/my_service:resume")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        service.resume()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = service.resume_async()
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


CUSTOM_METHODS = [
    ("endpoints", "get_endpoints", ()),
    ("containers", "get_containers", ()),
    ("instances", "get_instances", ()),
    ("roles", "get_roles", ()),
    ("roles/my_role/grants-of", "iter_grants_of_service_role", ("my_role",)),
    ("roles/my_role/grants", "iter_grants_to_service_role", ("my_role",)),
]


@pytest.mark.parametrize("method, fn, fn_args", CUSTOM_METHODS)
def test_custom_methods(fake_root, service, method, fn, fn_args):
    args = (fake_root, "GET", BASE_URL + f"/databases/my_db/schemas/my_schema/services/my_service/{method}")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        it = getattr(service, fn)(*fn_args)
        assert list(it) == []
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        op = getattr(service, fn + "_async")(*fn_args)
        assert isinstance(op, PollingOperation)
        it = op.result()
        assert list(it) == []
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_get_service_status(fake_root, service):
    args = (fake_root, "GET", BASE_URL + "/databases/my_db/schemas/my_schema/services/my_service/status?timeout=0")
    kwargs = extra_params(query_params=[("timeout", 0)])

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response(FetchServiceStatus200Response().to_json())
        service.get_service_status()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response(FetchServiceStatus200Response().to_json())
        op = service.get_service_status_async()
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_get_service_logs(fake_root, service):
    args = (
        fake_root,
        "GET",
        BASE_URL
        + "/databases/my_db/schemas/my_schema/services/my_service/logs?"
        + "instanceId=1&containerName=my_container",
    )
    kwargs = extra_params(query_params=[("instanceId", 1), ("containerName", "my_container")])

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response(FetchServiceLogs200Response().to_json())
        service.get_service_logs(1, "my_container")
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response(FetchServiceLogs200Response().to_json())
        op = service.get_service_logs_async(1, "my_container")
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)



import pytest

from snowflake.core.api_integration import ApiIntegration, AwsHook, AzureHook, GitHook, GoogleCloudHook
from tests.integ.utils import random_string


@pytest.mark.prodlike_only
@pytest.mark.skip_gov
def test_create_and_fetch_aws(api_integrations):
    ai_name = random_string(10, "test_api_integration_create_")
    ai_def = ApiIntegration(
        name=ai_name,
        api_hook=AwsHook(
            api_provider="AWS_API_GATEWAY",
            api_aws_role_arn="arn:aws:iam::123456789012:role/fake_role",
            api_key="secret",
        ),
        api_allowed_prefixes=["https://xyz.execute-api.us-west-2.amazonaws.com/test"],
        enabled=True,
        comment="created by test_create_and_fetch_aws",
    )
    ai = api_integrations.create(ai_def)
    try:
        fetched_ai = ai.fetch()
        assert fetched_ai.name == ai_name.upper()
        assert isinstance(fetched_ai.api_hook, AwsHook)
        assert fetched_ai.api_hook.api_provider == ai_def.api_hook.api_provider
        assert fetched_ai.api_hook.api_aws_role_arn == ai_def.api_hook.api_aws_role_arn
        assert fetched_ai.api_hook.api_key is not None
        assert fetched_ai.api_allowed_prefixes == ai_def.api_allowed_prefixes
        assert fetched_ai.enabled is ai_def.enabled
        assert fetched_ai.api_blocked_prefixes is None
        assert fetched_ai.comment == ai_def.comment
    finally:
        ai.drop()


@pytest.mark.prodlike_only
def test_create_and_fetch_azure(api_integrations):
    ai_name = random_string(10, "test_api_integration_create_")
    ai_def = ApiIntegration(
        name=ai_name,
        api_hook=AzureHook(
            api_provider="AZURE_API_MANAGEMENT",
            azure_tenant_id="azsnowtestoutlook.onmicrosoft.com",
            azure_ad_application_id="dummy_app_id",
        ),
        api_allowed_prefixes=["https://ext-func-test.azure-api.net/echo"],
        enabled=True,
        comment="created by test_create_and_fetch_azure",
    )
    ai = api_integrations.create(ai_def)
    try:
        fetched_ai = ai.fetch()
        assert fetched_ai.name == ai_name.upper()
        assert isinstance(fetched_ai.api_hook, AzureHook)
        assert fetched_ai.api_hook.api_provider == ai_def.api_hook.api_provider
        assert fetched_ai.api_hook.azure_tenant_id == ai_def.api_hook.azure_tenant_id
        assert fetched_ai.api_hook.azure_ad_application_id == ai_def.api_hook.azure_ad_application_id
        assert fetched_ai.api_hook.api_key is None
        assert fetched_ai.api_allowed_prefixes == ai_def.api_allowed_prefixes
        assert fetched_ai.enabled is ai_def.enabled
        assert fetched_ai.api_blocked_prefixes is None
        assert fetched_ai.comment == ai_def.comment
    finally:
        ai.drop()


@pytest.mark.prodlike_only
def test_create_and_fetch_gc(api_integrations):
    ai_name = random_string(10, "test_api_integration_create_")
    ai_def = ApiIntegration(
        name=ai_name,
        api_hook=GoogleCloudHook(api_provider="GOOGLE_API_GATEWAY", google_audience="dummy_aud"),
        api_allowed_prefixes=["https://fake.uc.gateway.dev"],
        enabled=True,
        comment="created by test_create_and_fetch_gc",
    )
    ai = api_integrations.create(ai_def)
    try:
        fetched_ai = ai.fetch()
        assert fetched_ai.name == ai_name.upper()
        assert isinstance(fetched_ai.api_hook, GoogleCloudHook)
        assert fetched_ai.api_hook.api_provider == ai_def.api_hook.api_provider
        assert fetched_ai.api_hook.google_audience == ai_def.api_hook.google_audience
        assert fetched_ai.api_hook.api_key is None
        assert fetched_ai.api_allowed_prefixes == ai_def.api_allowed_prefixes
        assert fetched_ai.enabled is ai_def.enabled
        assert fetched_ai.api_blocked_prefixes is None
        assert fetched_ai.comment == ai_def.comment
    finally:
        ai.drop()


def test_create_and_fetch_git(api_integrations):
    ai_name = random_string(10, "test_api_integration_create_")
    ai_def = ApiIntegration(
        name=ai_name,
        api_hook=GitHook(allow_any_secret=True),
        api_allowed_prefixes=["https://github.com"],
        enabled=False,
        api_blocked_prefixes=["https://github.com/snowflakedb/snowpy"],
        comment="created by test_create_and_fetch_git",
    )
    ai = api_integrations.create(ai_def)
    try:
        fetched_ai = ai.fetch()
        assert fetched_ai.name == ai_name.upper()
        assert isinstance(fetched_ai.api_hook, GitHook)
        assert fetched_ai.api_hook.allow_any_secret is ai_def.api_hook.allow_any_secret
        assert fetched_ai.api_hook.allowed_authentication_secrets is None
        assert fetched_ai.api_hook.allowed_api_authentication_integrations is None
        assert fetched_ai.api_allowed_prefixes == ai_def.api_allowed_prefixes
        assert fetched_ai.enabled is ai_def.enabled
        assert fetched_ai.api_blocked_prefixes == ai_def.api_blocked_prefixes
        assert fetched_ai.comment == ai_def.comment
    finally:
        ai.drop()



"""Manages virtual Warehouses.

Example:
    >>> warehouse_name = "MYWAREHOUSE"

Refer to :class:`snowflake.core.Root` to create the ``root``.
"""

from ._warehouse import Warehouse, WarehouseCollection, WarehouseResource


__all__ = ["Warehouse", "WarehouseCollection", "WarehouseResource"]



import base64

import pytest

from spark.connect.base_pb2 import AnalyzePlanRequest, ConfigRequest, ExecutePlanRequest, KeyValue, UserContext
from spark.connect.envelope_pb2 import ResponseEnvelope

from snowflake.core.spark_connect import SparkConnectResource


TEST_CONNECTIONS = ["connection_default", "pat_with_external_session_connection"]


def _block_for_result_if_necessary(resp_bytes: bytearray, connection):
    assert resp_bytes is not None
    response_envelope = ResponseEnvelope()
    response_envelope.ParseFromString(bytes(resp_bytes))
    if response_envelope.query_id:
        # query took long (> 45s) time and GS returned status URL with query id to pole
        cursor = connection.cursor()
        cursor.get_results_from_sfqid(response_envelope.query_id)
        # Block till the result is ready
        cursor._prefetch_hook()
        # Read the result back
        data = cursor._result_set.batches[0]._data
        resp_bytes = base64.b64decode(data)
        response_envelope = ResponseEnvelope()
        response_envelope.ParseFromString(resp_bytes)
    return response_envelope


@pytest.mark.flaky
@pytest.mark.skip_notebook
@pytest.mark.skip_storedproc
@pytest.mark.skip_gov
@pytest.mark.min_sf_ver("9.18.0")
@pytest.mark.parametrize("spark_connection", TEST_CONNECTIONS, indirect=True)
def test_execute_plan(spark_connection, external_session_id):
    exec_plan_request = ExecutePlanRequest(
        session_id=external_session_id,
        user_context=UserContext(user_id="test", user_name="ssa"),
        operation_id="test_grpc_over_rest",
        client_type="python API",
    )
    spark_connect_resource = SparkConnectResource(spark_connection)
    response = spark_connect_resource.execute_plan(exec_plan_request.SerializeToString())
    response_envelope = _block_for_result_if_necessary(response, spark_connection)
    assert response_envelope.WhichOneof("response_type") == "execute_plan_response"
    assert response_envelope.execute_plan_response.session_id == exec_plan_request.session_id
    assert response_envelope.execute_plan_response.result_complete is not None


@pytest.mark.flaky
@pytest.mark.skip_notebook
@pytest.mark.skip_storedproc
@pytest.mark.skip_gov
@pytest.mark.min_sf_ver("9.18.0")
@pytest.mark.parametrize("spark_connection", TEST_CONNECTIONS, indirect=True)
def test_analyze_plan(spark_connection, external_session_id):
    analyze_plan_request = AnalyzePlanRequest(
        session_id=external_session_id,
        user_context=UserContext(user_id="test", user_name="ssa"),
        spark_version=AnalyzePlanRequest.SparkVersion(),
        client_type="python API",
    )
    spark_connect_resource = SparkConnectResource(spark_connection)
    response = spark_connect_resource.analyze_plan(analyze_plan_request.SerializeToString())
    response_envelope = _block_for_result_if_necessary(response, spark_connection)
    assert response_envelope.WhichOneof("response_type") == "analyze_plan_response"
    assert response_envelope.analyze_plan_response.session_id == analyze_plan_request.session_id


@pytest.mark.flaky
@pytest.mark.skip_notebook
@pytest.mark.skip_storedproc
@pytest.mark.skip_gov
@pytest.mark.min_sf_ver("9.18.0")
@pytest.mark.parametrize("spark_connection", TEST_CONNECTIONS, indirect=True)
def test_config(spark_connection, external_session_id):
    config_request = ConfigRequest(
        session_id=external_session_id,
        user_context=UserContext(user_id="test", user_name="ssa"),
        client_type="python API",
        operation=ConfigRequest.Operation(set=ConfigRequest.Set(pairs=[KeyValue(key="foo", value="foo42")])),
    )
    spark_connect_resource = SparkConnectResource(spark_connection)
    response = spark_connect_resource.config(config_request.SerializeToString())
    response_envelope = _block_for_result_if_necessary(response, spark_connection)
    assert response_envelope.WhichOneof("response_type") == "config_response"
    assert response_envelope.config_response.session_id == config_request.session_id


@pytest.mark.flaky
@pytest.mark.skip_notebook
@pytest.mark.skip_storedproc
@pytest.mark.skip_gov
@pytest.mark.min_sf_ver("9.18.0")
def test_same_session_for_pat_with_ext_session_id(
    pat_with_external_session_connection, connection, external_session_id
):
    spark_connect_resource = SparkConnectResource(pat_with_external_session_connection)

    # Set config
    config_request = ConfigRequest(
        session_id=external_session_id,
        user_context=UserContext(user_id="test", user_name="ssa"),
        client_type="python API",
        operation=ConfigRequest.Operation(set=ConfigRequest.Set(pairs=[KeyValue(key="foo", value="foo42")])),
    )
    response = spark_connect_resource.config(config_request.SerializeToString())
    response_envelope = _block_for_result_if_necessary(response, connection)
    assert response_envelope.WhichOneof("response_type") == "config_response"
    assert response_envelope.config_response.session_id == config_request.session_id

    # Read the set config back
    config_request = ConfigRequest(
        session_id=external_session_id,
        user_context=UserContext(user_id="test", user_name="ssa"),
        client_type="python API",
        operation=ConfigRequest.Operation(get=ConfigRequest.Get(keys=["foo"])),
    )
    response = spark_connect_resource.config(config_request.SerializeToString())
    response_envelope = _block_for_result_if_necessary(response, connection)
    assert response_envelope.WhichOneof("response_type") == "config_response"
    assert response_envelope.config_response.session_id == config_request.session_id
    assert response_envelope.config_response.pairs is not None
    key_val = list(response_envelope.config_response.pairs)
    assert len(key_val) == 1
    assert key_val[0].key == "foo"
    assert key_val[0].value == "foo42"



from typing import Optional

import pytest

from snowflake.core.notification_integration import NotificationEmail, NotificationIntegration
from snowflake.core.tag import TagResource, TagValue

from ..base_tag_tests import BaseTagTests
from ..utils import random_string


pytestmark = [
    pytest.mark.internal_only,
    pytest.mark.use_accountadmin,
]


class TestNotificationIntegrationTags(BaseTagTests):
    @pytest.fixture(autouse=True)
    def setup(self, notification_integrations, set_internal_params):
        self.notification_integrations = notification_integrations
        integration_name = random_string(6, "notification_integration_for_tag_tests_")
        integration = NotificationIntegration(
            name=integration_name,
            notification_hook=NotificationEmail(
                allowed_recipients=["test1@snowflake.com", "test2@snowflake.com"],
                default_recipients=["test1@snowflake.com"],
                default_subject="test default subject",
            ),
            enabled=True,
        )
        with set_internal_params({"ENABLE_LIMIT_RECIPIENTS_TO_SENDING_ACCOUNT": False}):
            self.integration_res = notification_integrations.create(integration)
        try:
            yield
        finally:
            self.integration_res.drop(if_exists=True)

    @property
    def resource_level_name(self) -> str:
        return "INTEGRATION"

    def set_tags(
        self, tags: dict[TagResource, TagValue], if_exists: Optional[bool] = False, resource_name: Optional[str] = None
    ):
        self.notification_integrations[resource_name or self.integration_res.name].set_tags(tags, if_exists)

    def unset_tags(
        self, tag_resources: set[TagResource], if_exists: Optional[bool] = False, resource_name: Optional[str] = None
    ):
        self.notification_integrations[resource_name or self.integration_res.name].unset_tags(tag_resources, if_exists)

    def get_tags(
        self, with_lineage: Optional[bool] = False, resource_name: Optional[str] = None
    ) -> dict[TagResource, TagValue]:
        return self.notification_integrations[resource_name or self.integration_res.name].get_tags(with_lineage)



"""
Snowflake Table API.

The Snowflake Table API is a REST API that you can use to access, update, and perform certain actions on Tables resource in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any, List, Optional

from pydantic import BaseModel, ConfigDict, StrictStr

from snowflake.core.table._generated.models.table_column import TableColumn, TableColumnModel


class TableAsSelect(BaseModel):
    """A model object representing the TableAsSelect resource.

    Constructs an object of type TableAsSelect with the provided properties.

    Parameters
    __________
    name : str, optional
        Specifies the name for the table, must be unique for the schema in which the table is created
    columns : list[TableColumn], optional

    cluster_by : list[str], optional
        Specifies one or more columns or column expressions in the table as the clustering key
    """

    name: Optional[StrictStr] = None

    columns: Optional[List[TableColumn]] = None

    cluster_by: Optional[List[StrictStr]] = None

    __properties = ["name", "columns", "cluster_by"]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> TableAsSelect:
        """Create an instance of TableAsSelect from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        # override the default output from pydantic by calling `to_dict()` of each item in columns (list)
        _items = []
        if self.columns:
            for _item in self.columns:
                if _item:
                    _items.append(_item.to_dict())
            _dict["columns"] = _items

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> TableAsSelect:
        """Create an instance of TableAsSelect from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return TableAsSelect.model_validate(obj)

        _obj = TableAsSelect.model_validate(
            {
                "name": obj.get("name"),
                "columns": [TableColumn.from_dict(_item) for _item in obj.get("columns")]
                if obj.get("columns") is not None
                else None,
                "cluster_by": obj.get("cluster_by"),
            }
        )

        return _obj


class TableAsSelectModel:
    def __init__(
        self,  # optional properties
        name: Optional[str] = None,
        columns: Optional[list[TableColumn]] = None,
        cluster_by: Optional[list[str]] = None,
    ):
        """A model object representing the TableAsSelect resource.

        Constructs an object of type TableAsSelect with the provided properties.

        Parameters
        __________
        name : str, optional
            Specifies the name for the table, must be unique for the schema in which the table is created
        columns : list[TableColumn], optional

        cluster_by : list[str], optional
            Specifies one or more columns or column expressions in the table as the clustering key
        """
        self.name = name
        self.columns = columns
        self.cluster_by = cluster_by

    __properties = ["name", "columns", "cluster_by"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return TableAsSelect(
            name=self.name,
            columns=[x._to_model() for x in self.columns] if self.columns is not None else None,
            cluster_by=self.cluster_by,
        )

    @classmethod
    def _from_model(cls, model) -> TableAsSelectModel:
        return TableAsSelectModel(
            name=model.name,
            columns=[TableColumnModel._from_model(x) for x in model.columns] if model.columns else None,
            cluster_by=model.cluster_by,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a TableAsSelect.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the TableAsSelect object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> TableAsSelectModel:
        """Create an instance of TableAsSelect from a dict.

        This method constructs a TableAsSelect object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        TableAsSelect
            A TableAsSelect object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(TableAsSelect.from_dict(obj))


TableAsSelect._model_class = TableAsSelectModel



"""
Cortex Inference API.

OpenAPI 3.0 specification for the Cortex REST API  # noqa: E501

The version of the OpenAPI document: 0.1.0
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any, Optional

from pydantic import BaseModel, ConfigDict, StrictStr

from snowflake.core.cortex.inference_service._generated.models.tool_tool_spec_input_schema import (
    ToolToolSpecInputSchema,
)


class ToolToolSpec(BaseModel):
    """A model object representing the ToolToolSpec resource.

    Constructs an object of type ToolToolSpec with the provided properties.

    Parameters
    __________
    type : str
        The type of tool capability. Can be specialized types like
        'cortex_analyst_text_to_sql' or 'generic' for general-purpose tools.
    name : str
        Unique identifier for referencing this tool instance.
        Used to match with configuration in tool_resources.
    description : str, optional
        Description of the tool to be considered for tool use
    input_schema : ToolToolSpecInputSchema, optional
    """

    type: StrictStr

    name: StrictStr

    description: Optional[StrictStr] = None

    input_schema: Optional[ToolToolSpecInputSchema] = None

    __properties = ["type", "name", "description", "input_schema"]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> ToolToolSpec:
        """Create an instance of ToolToolSpec from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        # override the default output from pydantic by calling `to_dict()` of input_schema
        if self.input_schema:
            _dict["input_schema"] = self.input_schema.to_dict()

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> ToolToolSpec:
        """Create an instance of ToolToolSpec from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return ToolToolSpec.model_validate(obj)

        _obj = ToolToolSpec.model_validate(
            {
                "type": obj.get("type"),
                "name": obj.get("name"),
                "description": obj.get("description"),
                "input_schema": ToolToolSpecInputSchema.from_dict(obj.get("input_schema"))
                if obj.get("input_schema") is not None
                else None,
            }
        )

        return _obj


class ToolToolSpecModel:
    def __init__(
        self,
        type: str,
        name: str,
        # optional properties
        description: Optional[str] = None,
        input_schema: Optional[ToolToolSpecInputSchema] = None,
    ):
        """A model object representing the ToolToolSpec resource.

                Constructs an object of type ToolToolSpec with the provided properties.

                Parameters
                __________
                type : str
                    The type of tool capability. Can be specialized types like
        'cortex_analyst_text_to_sql' or 'generic' for general-purpose tools.

                name : str
                    Unique identifier for referencing this tool instance.
        Used to match with configuration in tool_resources.

                description : str, optional
                    Description of the tool to be considered for tool use
                input_schema : ToolToolSpecInputSchema, optional
        """
        self.type = type
        self.name = name
        self.description = description
        self.input_schema = input_schema

    __properties = ["type", "name", "description", "input_schema"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return ToolToolSpec(
            type=self.type,
            name=self.name,
            description=self.description,
            input_schema=self.input_schema,
        )

    @classmethod
    def _from_model(cls, model) -> ToolToolSpecModel:
        return ToolToolSpecModel(
            type=model.type,
            name=model.name,
            description=model.description,
            input_schema=model.input_schema,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a ToolToolSpec.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the ToolToolSpec object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> ToolToolSpecModel:
        """Create an instance of ToolToolSpec from a dict.

        This method constructs a ToolToolSpec object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        ToolToolSpec
            A ToolToolSpec object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(ToolToolSpec.from_dict(obj))


ToolToolSpec._model_class = ToolToolSpecModel



import pytest

from snowflake.core.user import Securable


pytestmark = pytest.mark.min_sf_ver("8.39.0")


@pytest.mark.use_accountadmin
def test_iter_grant(users, test_user_name, test_role_name, test_role_name_2):
    users[test_user_name].grant_role(role_type="ROLE", role=Securable(name=test_role_name))

    assert len(list(users[test_user_name].iter_grants_to())) == 1

    users[test_user_name].grant_role(role_type="ROLE", role=Securable(name=test_role_name_2))

    assert len(list(users[test_user_name].iter_grants_to())) == 2

    users[test_user_name].revoke_role(role_type="ROLE", role=Securable(name=test_role_name_2))

    assert len(list(users[test_user_name].iter_grants_to())) == 1



"""
Snowflake Table API.

The Snowflake Table API is a REST API that you can use to access, update, and perform certain actions on Tables resource in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any, List, Optional

from pydantic import BaseModel, ConfigDict, StrictBool, StrictInt, StrictStr

from snowflake.core.table._generated.models.constraint import Constraint, ConstraintModel


class TableColumn(BaseModel):
    """A model object representing the TableColumn resource.

    Constructs an object of type TableColumn with the provided properties.

    Parameters
    __________
    name : str
        Column name
    datatype : str
        The data type for the column
    nullable : bool,  default True
        Specifies that the column does allow NULL values or not.
    collate : str, optional
        Specifies the collation to use for column operations such as string comparison
    default : str, optional
        Specifies whether a default value is automatically inserted in the column if a value is not explicitly specified via an INSERT or CREATE TABLE AS SELECT statement
    autoincrement : bool, optional

    autoincrement_start : int, optional
        The default value for the column starts with the specified number
    autoincrement_increment : int, optional
        Each successive value for the column automatically increments by the specified amount
    constraints : list[Constraint], optional

    comment : str, optional
        Specifies a comment for the column
    """

    name: StrictStr

    datatype: StrictStr

    nullable: Optional[StrictBool] = True

    collate: Optional[StrictStr] = None

    default: Optional[StrictStr] = None

    autoincrement: Optional[StrictBool] = None

    autoincrement_start: Optional[StrictInt] = None

    autoincrement_increment: Optional[StrictInt] = None

    constraints: Optional[List[Constraint]] = None

    comment: Optional[StrictStr] = None

    __properties = [
        "name",
        "datatype",
        "nullable",
        "collate",
        "default",
        "autoincrement",
        "autoincrement_start",
        "autoincrement_increment",
        "constraints",
        "comment",
    ]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> TableColumn:
        """Create an instance of TableColumn from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        # override the default output from pydantic by calling `to_dict()` of each item in constraints (list)
        _items = []
        if self.constraints:
            for _item in self.constraints:
                if _item:
                    _items.append(_item.to_dict())
            _dict["constraints"] = _items

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> TableColumn:
        """Create an instance of TableColumn from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return TableColumn.model_validate(obj)

        _obj = TableColumn.model_validate(
            {
                "name": obj.get("name"),
                "datatype": obj.get("datatype"),
                "nullable": obj.get("nullable") if obj.get("nullable") is not None else True,
                "collate": obj.get("collate"),
                "default": obj.get("default"),
                "autoincrement": obj.get("autoincrement"),
                "autoincrement_start": obj.get("autoincrement_start"),
                "autoincrement_increment": obj.get("autoincrement_increment"),
                "constraints": [Constraint.from_dict(_item) for _item in obj.get("constraints")]
                if obj.get("constraints") is not None
                else None,
                "comment": obj.get("comment"),
            }
        )

        return _obj


class TableColumnModel:
    def __init__(
        self,
        name: str,
        datatype: str,
        # optional properties
        nullable: Optional[bool] = True,
        collate: Optional[str] = None,
        default: Optional[str] = None,
        autoincrement: Optional[bool] = None,
        autoincrement_start: Optional[int] = None,
        autoincrement_increment: Optional[int] = None,
        constraints: Optional[list[Constraint]] = None,
        comment: Optional[str] = None,
    ):
        """A model object representing the TableColumn resource.

        Constructs an object of type TableColumn with the provided properties.

        Parameters
        __________
        name : str
            Column name
        datatype : str
            The data type for the column
        nullable : bool,  default True
            Specifies that the column does allow NULL values or not.
        collate : str, optional
            Specifies the collation to use for column operations such as string comparison
        default : str, optional
            Specifies whether a default value is automatically inserted in the column if a value is not explicitly specified via an INSERT or CREATE TABLE AS SELECT statement
        autoincrement : bool, optional

        autoincrement_start : int, optional
            The default value for the column starts with the specified number
        autoincrement_increment : int, optional
            Each successive value for the column automatically increments by the specified amount
        constraints : list[Constraint], optional

        comment : str, optional
            Specifies a comment for the column
        """
        self.name = name
        self.datatype = datatype
        self.nullable = nullable
        self.collate = collate
        self.default = default
        self.autoincrement = autoincrement
        self.autoincrement_start = autoincrement_start
        self.autoincrement_increment = autoincrement_increment
        self.constraints = constraints
        self.comment = comment

    __properties = [
        "name",
        "datatype",
        "nullable",
        "collate",
        "default",
        "autoincrement",
        "autoincrement_start",
        "autoincrement_increment",
        "constraints",
        "comment",
    ]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return TableColumn(
            name=self.name,
            datatype=self.datatype,
            nullable=self.nullable,
            collate=self.collate,
            default=self.default,
            autoincrement=self.autoincrement,
            autoincrement_start=self.autoincrement_start,
            autoincrement_increment=self.autoincrement_increment,
            constraints=[x._to_model() for x in self.constraints] if self.constraints is not None else None,
            comment=self.comment,
        )

    @classmethod
    def _from_model(cls, model) -> TableColumnModel:
        return TableColumnModel(
            name=model.name,
            datatype=model.datatype,
            nullable=model.nullable,
            collate=model.collate,
            default=model.default,
            autoincrement=model.autoincrement,
            autoincrement_start=model.autoincrement_start,
            autoincrement_increment=model.autoincrement_increment,
            constraints=[ConstraintModel._from_model(x) for x in model.constraints] if model.constraints else None,
            comment=model.comment,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a TableColumn.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the TableColumn object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> TableColumnModel:
        """Create an instance of TableColumn from a dict.

        This method constructs a TableColumn object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        TableColumn
            A TableColumn object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(TableColumn.from_dict(obj))


TableColumn._model_class = TableColumnModel



from typing import TYPE_CHECKING

from snowflake.core import PollingOperation
from snowflake.core._generated.api_client import StoredProcApiClient
from snowflake.core._internal.telemetry import api_telemetry
from snowflake.core.cortex.inference_service._generated.api import CortexInferenceApi
from snowflake.core.cortex.inference_service._generated.models import CompleteRequest
from snowflake.core.rest import SSEClient


if TYPE_CHECKING:
    from snowflake.core import Root


class CortexInferenceService:
    """Represents the Snowflake Cortex Inference Service resource."""

    def __init__(self, root: "Root") -> None:
        self._api = CortexInferenceApi(root=root, resource_class=None, sproc_client=StoredProcApiClient(root=root))

    @api_telemetry
    def complete(self, complete_request: CompleteRequest) -> SSEClient:
        """Perform LLM text completion inference, similar to snowflake.cortex.Complete.

        Parameters
        __________
        complete_request: CompleteRequest
            LLM text completion request.
        """
        return SSEClient(
            self._api.cortex_llm_inference_complete(complete_request, async_req=False, _preload_content=False)
        )

    @api_telemetry
    def complete_async(self, complete_request: CompleteRequest) -> PollingOperation[SSEClient]:
        """An asynchronous version of :func:`complete`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self._api.cortex_llm_inference_complete(complete_request, async_req=True, _preload_content=False)
        return PollingOperation(future, lambda x: SSEClient(x))



import copy

import pytest

from tests.integ.utils import random_string

from .conftest import test_password_policy_template


@pytest.fixture(scope="module")
def password_policies_extended(password_policies):
    names_list = []

    for _ in range(5):
        names_list.append(random_string(10, "test_password_policy_iter_a_"))

    for _ in range(7):
        names_list.append(random_string(10, "test_password_policy_iter_b_"))

    for _ in range(3):
        names_list.append(random_string(10, "test_password_policy_iter_c_"))

    try:
        for name in names_list:
            password_policy = copy.deepcopy(test_password_policy_template)
            password_policy.name = name
            password_policies.create(password_policy)

        yield password_policies
    finally:
        for name in names_list:
            password_policies[name].drop(if_exists=True)


def test_iter_raw(password_policies_extended):
    assert len(list(password_policies_extended.iter())) >= 15


def test_iter_like(password_policies_extended):
    assert len(list(password_policies_extended.iter(like="test_password_policy_iter_a_%"))) == 5
    assert len(list(password_policies_extended.iter(like="test_password_policy_iter_b_%"))) == 7
    assert len(list(password_policies_extended.iter(like="test_password_policy_iter_c_%"))) == 3
    assert len(list(password_policies_extended.iter(like="TEST_PASSWORD_POLICY_ITER_C_%"))) == 3
    assert len(list(password_policies_extended.iter(like="nonexistent_pattern_%"))) == 0


def test_iter_limit(password_policies_extended):
    assert len(list(password_policies_extended.iter(limit=2))) == 2


def test_iter_starts_with(password_policies_extended):
    assert len(list(password_policies_extended.iter(starts_with="test_password_policy_iter_a_".upper()))) == 5
    assert len(list(password_policies_extended.iter(starts_with="test_password_policy_iter_a_"))) == 0
    assert len(list(password_policies_extended.iter(starts_with="test_password_policy_iter_d_".upper()))) == 0



from collections.abc import Iterable
from typing import TYPE_CHECKING, Optional

from pydantic import StrictStr

from snowflake.core._common import DeleteMode
from snowflake.core._generated.api_client import StoredProcApiClient
from snowflake.core.grant._generated import GrantApi
from snowflake.core.grant._generated.models import Grant as GrantModel
from snowflake.core.grant._generated.models.grant import GrantModel as GrantModel_generated
from snowflake.core.grant._grant import Grant
from snowflake.core.grant._grantee import Grantee


if TYPE_CHECKING:
    from snowflake.core import Root


class Grants:
    """The entry point of the Snowflake Core Python APIs to manage Snowflake Grants."""

    def __init__(self, root: "Root") -> None:
        self._root = root
        self._api = GrantApi(root=self._root, resource_class=None, sproc_client=StoredProcApiClient(root=self._root))

    def grant(self, grant: Grant) -> None:
        """
        Grant the specified privilege(s) on the named securable to the named grantee.

        Parameters
        __________
        grant: Grant
            The ``Grant`` object, together with the ``Grant``'s properties:
            grantee, securable, privilege; grant_option is optional.

        Examples
        ________
        Applying a grant to a test role:

        >>> root.grants.grant(
        ...     Grant(
        ...         grantee=Grantees.role(name=role_name),
        ...         securable=Securables.current_account,
        ...         privileges=[Privileges.create_database],
        ...     )
        ... )
        """
        privileges = [privilege.value for privilege in grant.privileges] if grant.privileges else None
        request_body = GrantModel(privileges=privileges, grant_option=grant.grant_option)

        if grant.securable.scope is not None:
            self._api.grant_group_privilege(
                grantee_type=grant.grantee.grantee_type,
                grantee_name=grant.grantee.name,
                bulk_grant_type=grant.securable.name,
                securable_type_plural=grant.securable.securable_type,
                scope_type=grant.securable.scope.securable_type,
                scope_name=grant.securable.scope.name,
                grant=request_body,
            )
        else:
            self._api.grant_privilege(
                grantee_type=grant.grantee.grantee_type,
                grantee_name=grant.grantee.name,
                securable_type=grant.securable.securable_type,
                securable_name=grant.securable.name,
                grant=request_body,
            )

    def revoke(self, grant: Grant, mode: DeleteMode = DeleteMode.restrict) -> None:
        """
        Revoke the specified privilege(s) on the named securable to the named grantee.

        Parameters
        __________
        grant: Grant
            The ``Grant`` object, together with the ``Grant``'s properties:
            grantee, securable, privilege; grant_option is optional.

        Examples
        ________
        Revoking a Privilege from test role:

        >>> root.grants.revoke(
        ...     Grant(
        ...         grantee=Grantees.role(name=role_name),
        ...         securable=Securables.current_account,
        ...         privileges=[Privileges.create_database],
        ...     )
        ... )
        """
        privileges = [privilege.value for privilege in grant.privileges] if grant.privileges else []

        real_mode = mode.value
        for privilege in privileges:
            if grant.securable.scope is not None:
                self._api.revoke_group_privilege(
                    grantee_type=grant.grantee.grantee_type,
                    grantee_name=grant.grantee.name,
                    bulk_grant_type=grant.securable.name,
                    securable_type_plural=grant.securable.securable_type,
                    scope_type=grant.securable.scope.securable_type,
                    scope_name=grant.securable.scope.name,
                    privilege=privilege,
                    delete_mode=StrictStr(real_mode),
                )
            else:
                self._api.revoke_privilege(
                    grantee_type=grant.grantee.grantee_type,
                    grantee_name=grant.grantee.name,
                    securable_type=grant.securable.securable_type,
                    securable_name=grant.securable.name,
                    privilege=privilege,
                    delete_mode=StrictStr(real_mode),
                )

    def revoke_grant_option(self, grant: Grant, mode: DeleteMode = DeleteMode.restrict) -> None:
        """
        Revoke the grant option on the specified privilege(s) on the named securable to the named grantee.

        Parameters
        __________
        grant: Grant
            The ``Grant`` object, together with the ``Grant``'s properties:
            grantee, securable, privilege; grant_option is optional.

        Examples
        ________
        Revoking grant option for a Privilege from test role:

        >>> root.grants.revoke(
        ...     Grant(
        ...         grantee=Grantees.role(name=role_name),
        ...         securable=Securables.current_account,
        ...         privileges=[Privileges.create_database],
        ...     )
        ... )
        """
        privileges = [privilege.value for privilege in grant.privileges] if grant.privileges else []

        real_mode = mode.value
        for privilege in privileges:
            if grant.securable.scope is not None:
                self._api.revoke_group_privilege_grant_option(
                    grantee_type=grant.grantee.grantee_type,
                    grantee_name=grant.grantee.name,
                    bulk_grant_type=grant.securable.name,
                    securable_type_plural=grant.securable.securable_type,
                    scope_type=grant.securable.scope.securable_type,
                    scope_name=grant.securable.scope.name,
                    privilege=privilege,
                    delete_mode=StrictStr(real_mode),
                )
            else:
                self._api.revoke_privilege_grant_option(
                    grantee_type=grant.grantee.grantee_type,
                    grantee_name=grant.grantee.name,
                    securable_type=grant.securable.securable_type,
                    securable_name=grant.securable.name,
                    privilege=privilege,
                    delete_mode=StrictStr(real_mode),
                )

    def to(self, grantee: Grantee, limit: Optional[int] = None) -> Iterable[GrantModel_generated]:
        """
        List the roles and privileges granted to the specified grantee.

        Parameters
        __________
        grantee: Grantee
            The ``Grantee`` to list the roles and privileges of.

        Examples
        ________
        Listing the roles and privileges granted to the grantee:

        >>> root.grants.to(Grantee(name="test-user", grantee_type="user"), limit=10)
        """
        grants = self._api.list_grants_to(
            grantee_type=grantee.grantee_type, grantee_name=grantee.name, show_limit=limit
        )
        return map(GrantModel_generated._from_model, iter(grants))
